Package: idar
Version: 1.7
Date: 2026-04-13
Title: Individual Diversity-Area Relationships
Authors@R: c(person(given = "Marcelino",
                        family = "de la Cruz",
                        role = c("aut", "cre"),
                        email = "marcelino.delacruz@urjc.es",
                        comment = c(ORCID ="0000-0002-9080-4525")),
			person(given ="Etienne", 
                        family = "Laliberté",
                        role = c("ctb")))
Depends: picante, spatstat (>= 2.0-0)
Imports: ade4, ape, spatstat.geom, spatstat.explore, spatstat.random
Suggests: ecespa, vegan
Description: Computes and tests individual (species, phylogenetic and functional) diversity-area relationships, i.e., how species-, phylogenetic- and functional-diversity varies with spatial scale around the individuals of some species in a community. See applications of these methods in Wiegand et al. (2007) <doi:10.1073/pnas.0705621104> or Chacon-Labella et al. (2016) <doi:10.1007/s00442-016-3547-z>.
License: GPL (>= 2)
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-04-11 11:15:06 UTC; user
Author: Marcelino de la Cruz [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9080-4525>),
  Etienne Laliberté [ctb]
Maintainer: Marcelino de la Cruz <marcelino.delacruz@urjc.es>
Repository: CRAN
Date/Publication: 2026-04-14 12:00:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-15 05:20:05 UTC; unix
Archs: idar.so.dSYM

## compiled code
useDynLib(idar, .registration = TRUE)

# Export regular functions
export("controldis","envelope4idar","ifdar","ipscar","ipsear","ipsim","ipsimlist","ipsrar","ipsvar","isar","LF.gof","localdar","midar","mitable","pisar","proportion.idar","plot.pidar","risar","simulador2", "checktraits", "checktree","multifocalsimulator", gowdis, fdisp)

# imports
importFrom('spatstat.explore',  'density.ppp',  'fv')
importFrom('spatstat.random',  'rpoispp')
importFrom('spatstat.geom',  'area.owin',  'as.im',  'bdist.points',  'erosion',  'gridcentres',  'inside.owin',  'is.marked',  'is.ppp',  'marks',  'marks<-',  'owin',  'ppp',  'progressreport',  'unmark')
importFrom("ape", "is.ultrametric")
importFrom("ape", "vcv.phylo")

# import required by FD functions
importFrom("ade4", "bicenter.wt")

# ToDo: "importFrom" only required functions 
import(picante)

# eliminated  import(FD) because of FD getting archived

#remove when the dust settles:
import(spatstat)

## Registration of S3 methods
import(stats)
import(graphics)
S3method(plot, pidar)

  #fdis: function to compute IFDAR (IFDisp) --------------------------------------------------------------------
  # x is a community data.frame.
  # traits is a distance matrix (as a matrix, i.e., from as.matrix(dist(traits)) or as.matrix(gowdis(traits))

 fdis <- function(x, traits){
     sp.a <- colSums(x) > 0
    if (sum(sp.a) < 1) 
        return(NA)
    if (sum(sp.a) == 1) 
        return(0)
    if (sum(sp.a) > 1) {
        m.ok <- x[, sp.a]
        sp.trait.ok <- !is.na(match(rownames(traits), colnames(m.ok)))
        if (!is.null(dim(m.ok))) {
            com.G.0 <- rowSums(m.ok) > 0
            cosad <- as.dist(traits[sp.trait.ok, sp.trait.ok, 
                drop = FALSE])
            if (sum(cosad, na.rm = TRUE) == 0) 
                return(0)
            if (sum(sp.trait.ok) >= 2) {
                 com.G.02 <- rowSums(m.ok[,labels(cosad)]) > 0 
                cosaf <- fdisp(d = cosad, a = m.ok[com.G.02, labels(cosad), 
                  drop = FALSE], tol = 1e-07)$FDis
            }
            if (sum(sp.trait.ok) < 2) {
                cosaf <- fdisp(d = cosad, a = t(as.matrix(m.ok[com.G.0, 
                  labels(cosad)])), tol = 1e-07)$FDis
            }
            media <- mean(cosaf, na.rm = T)
            result <- media
        }
        if (is.null(dim(m.ok))) 
            result <- NA
        return(result)
    }
}
   # ---- end function fdis ------------------------------------------------------------------------------------------------------------

###############################################################################
#
#    ISAR.R
#
#   Individual Species Area Relationship
#
#  $Version: 0.1 $  $Date: 2013/02/17 23:48:03 $
#
#   function to compute Indivdual Species Area Relationships (i.e., species
#   accumulation curves around individual species) in a community.
#
#    Value: an object of class isar, bassicaly a list with the following
#    elemnts:
#     r: vector of radii at which ISAR(r) has been estimated
#     isar: vector with the ISAR(r) values
#     npoints: number of focal points employed to estimate isar(r) at each 
#              radius r
#
#    Arguments:
#     mippp:    Multivariate point pattern (ppp of spatstat)
#     mippp.sp: Focal (unmarked) point pattern (ppp of spatstat) or character
#                string indicating one of the marks in the multivariate mippp 
#     mimark:   character string indicating one of the marks in the
#                multivariate mippp
#     namesark: name of the column with species names in the data.frame of
#                marks (only for multimarked ppp's)
#     r:        vector with the sequence of radii (>0) at which estimate
#                ISAR(r)
#     buffer:   a number indicating the width around the window of the focal 
#                point pattern that will be excluded from the computations of
#                ISAR(r) to control de edge effect, or the string "adapt", 
#                indicating that an adaptive border (of width ri) will be 
#                excluded for the computation of each ISAR(ri)
#     bfw:      an owin object indicating a subset of the focal point pattern 
#                that will be employed to compute ISAR(r)
#
#   Details:
#     btw can be employed to select different habitats and compute ISAR((R) i 
#      each habitat

 




 
isar<- function(mippp, mippp.sp=NULL, mimark=NULL,  namesmark=NULL, r=NULL,
	buffer=0, bfw=NULL) {


  #If species marks are within a data.frame of marks, get this column and 
  # discard the rest
  if(!is.null(namesmark)) mippp$marks <- factor((mippp$marks[namesmark][[1]]))  # OJO: que representa el [[1]] del final: es para coger los valores. si no se pone da problemas !!!!!!!!!!!!!!!!!!!!!!!!!!!
  #!!!!!!!!!!!!!!!!!!
  
  
  #If mippp.sp has not been provided as a ppp, build it from the multivariate
  # mippp and the provided mark
  if(!is.null(mippp.sp) & !is.ppp(mippp.sp)) {
   mippp.sp <- NULL
   mimark <- mippp.sp
  }

  if(!is.null(mimark)) if(mimark%in%levels(mippp$marks)==FALSE){
     stop(paste(mimark, " can't be recognized as a mark\n\n
     have you indicated in which column of thedataframe are the species
      marks? (argument 'namesmark'\n\n"))
      }
  if (is.null(mippp.sp)) mippp.sp<- mippp[mippp$marks==mimark]
       
  #If indicated, correct edge effect with a fixed buffer of size "buffer"
  # or with the window "bfw"
  if(buffer!="adapt"){	
    if(is.null(bfw)) bfw<- erosion( mippp$window, buffer)
    mippp.sp <- mippp.sp[inside.owin(mippp.sp, w=bfw)]
    npoints <- rep(mippp.sp$n, length(r))
    names(npoints) <- r
  }
       

  #Tabulate the number of individuals of each species (i.e. types) around each 
  # individual of the focal species within circles of each of the r radii
  
  
  cosamt <- mitable(mippp.sp, mippp, r)
	
  #If indicated correct edge effect with an adaptive buffer
  if(buffer=="adapt"){
    bdp <- bdist.points(mippp.sp)
    for (i in 1:length(r)) cosamt[[i]]<-cosamt[[i]][bdp>=r[i],]
    npoints <- sapply(cosamt, function(x) dim(x)[1])
  }
  	
  #Define bivariate emptines probability Ptj(0,r)
  # i.e., the probability of not finding species j within circles of radius r 
  # around the individuals of focal species.
  # x will be the vector of counts of specis j within circles of radius r 
  # around each individual of the focal species.
  Ptj <- function(x) sum(x==0)/length(x)

  #Define function to compute ISAR
  isar.r <- function(x){
    result <-sum(apply(x,2,function(x) 1-Ptj(x)))
    return(result)
  }
  
  #Compute ISAR(r) for each radius r
  isar <- sapply(cosamt, isar.r)

  # return results
  # result <- list(r=r, isar=isar, npoints=npoints)
   result <- data.frame(r=r, isar=isar)
  
   result <- fv(result, argu="r", ylab=substitute(ISAR(r), NULL),
              valu="isar", fmla=isar ~ r,
              alim=c(min(r),max(r)),
              labl=c("r", "%s(r)"),
              desc=c("radius of circle",
                     "%s"),
              fname="ISAR")

  
  
  
  #class(result)<-c("isar", class(result))
  return(result)
}
###############################################################################

###############################################################################
#
#    IFDAR.R
#
#   Individual Functional-Diversity -Area Relationship
#
#  $Version: 0.2 $  $Date: 2014/05/26 23:48:03 $
#
#
#   function to compute Indivdual Functional Diversity Area Relationships (i.e., acumulation of FDis curves
#    around individual species) in a community.
#
#    Value: an object of class fv , bassicaly a data.frame with the following
#    elemnts:
#     r: vector of radii at which IFDAR(r) has been estimated
#     ifdar: vector with the IFDAR(r) values
#     ###npoints: number of focal points employed to estimate isar(r) at each 
#              radius r
#
#    Arguments:
#     mippp:    Multivariate point pattern (ppp of spatstat)
#     mippp.sp: Focal (unmarked) point pattern (ppp of spatstat) or character
#                string indicating one of the marks in the multivariate mippp 
#     mimark:   character string indicating one of the marks in the
#                multivariate mippp
#     namesmark: name of the column with species names in the data.frame of
#                marks (only for multimarked ppp's)
#     traits:    A data.frame of traits with cols=traits, rows = species. rownames should be equal to species names in mippp 
#     r:        vector with the sequence of radii (>0) at which estimate
#                IFDAR(r)
#     buffer:   a number indicating the width around the window of the focal 
#                point pattern that will be excluded from the computations of
#                IFDAR(r) to control de edge effect, or the string "adapt", 
#                indicating that an adaptive border (of width ri) will be 
#                excluded for the computation of each ISAR(ri)
#     bfw:      an owin object indicating a subset of the focal point pattern 
#                that will be employed to compute ISAR(r)
#
#   Details:
#     bfw can be employed to select different habitats and compute IFDAR((R) in 
#      each habitat


 
ifdar<- function(mippp, mippp.sp=NULL, mimark=NULL,  namesmark=NULL, traits=NULL,
                      r=NULL, buffer=0, bfw=NULL, correct.trait.na=FALSE, correct.trait="mean") {
    if (is.null(traits))   stop("you should provide a dataframe of traits  to compute ifdar")
    
    #  if (is.data.frame(traits))  traits <- gowdis(traits)
    #  if (class(traits) == "dist")  traits <- as.matrix(traits)
    #  if (is.matrix(traits)) {
    #         if (dim(traits)[1] != dim(traits)[2])  traits <- as.matrix(gowdis(traits))
    #  }

    # Check mippp
    if (!is.null(namesmark))  mippp$marks <- factor((mippp$marks[namesmark][[1]]))

    # check traits
    idar <-"ifdar"
    traits <-checktraits(traits=traits, mippp=mippp, idar=idar, correct.trait.na=correct.trait.na, correct.trait=correct.trait)

    # Check mippp.sp
    if (!is.null(mippp.sp) & !is.ppp(mippp.sp)) {
        mippp.sp <- NULL
        mimark <- mippp.sp
    }
    if (!is.null(mimark)) 
        if (mimark %in% levels(mippp$marks) == FALSE) {
            stop(paste(mimark, " can't be recognized as a mark\n\n\n
                    have you indicated in which column of thedataframe are the species\n
                    marks? (argument 'namesmark'\n\n"))
        }
    if (is.null(mippp.sp)) 
        mippp.sp <- mippp[mippp$marks == mimark]
    if (buffer != "adapt") {
        if (is.null(bfw)) 
            bfw <- erosion(mippp$window, buffer)
        mippp.sp <- mippp.sp[inside.owin(mippp.sp, w = bfw)]
        npoints <- rep(mippp.sp$n, length(r))
        names(npoints) <- r
    }
    cosamt <- mitable(mippp.sp, mippp, r)
    if (buffer == "adapt") {
        bdp <- bdist.points(mippp.sp)
        for (i in 1:length(r)) cosamt[[i]] <- cosamt[[i]][bdp >= 
            r[i], ]
        npoints <- sapply(cosamt, function(x) dim(x)[1])
    }
    ifdar <- sapply(cosamt, function(x) fdis(x, traits = traits)) 
    result <- data.frame(r = r, ifdar = ifdar)
    result <- fv(result, argu = "r", ylab = substitute(IFDAR(r), 
        NULL), valu = "ifdar", fmla = ifdar ~ r, alim = c(min(r), 
        max(r)), labl = c("r", "%s(r)"), desc = c("radius of circle", 
        "%s"), fname = "IFDAR")
    return(result)
}
###############################################################################

midar<-
function (mippp, mippp.sp = NULL, mimark = NULL, namesmark = NULL, 
    traits = NULL, tree=NULL, r = NULL, buffer = 0, bfw = NULL, what=NULL) 
{
    if(!is.expression(what)& !is.function(what)) stop("'what' should be a valid R function or expression")
    
    # TODO: include a test that the expression "what" will accept a matrix and will return a unique value
   
    if (!is.null(namesmark)) 
        mippp$marks <- factor((mippp$marks[namesmark][[1]]))
   
   
    if (!is.null(mippp.sp) & !is.ppp(mippp.sp)) {
        mippp.sp <- NULL
        mimark <- mippp.sp
    }
    if (!is.null(mimark)) 
        if (mimark %in% levels(mippp$marks) == FALSE) {
            stop(paste(mimark, " can't be recognized as a mark\n\n\n\n                    have you indicated in which column of thedataframe are the species\n\n                    marks? (argument 'namesmark'\n\n"))
        }
    if (is.null(mippp.sp)) 
        mippp.sp <- mippp[mippp$marks == mimark]
    if (buffer != "adapt") {
        if (is.null(bfw)) 
            bfw <- erosion(mippp$window, buffer)
        mippp.sp <- mippp.sp[inside.owin(mippp.sp, w = bfw)]
        npoints <- rep(mippp.sp$n, length(r))
        names(npoints) <- r
    }
    cosamt <- mitable(mippp.sp, mippp, r)
    if (buffer == "adapt") {
        bdp <- bdist.points(mippp.sp)
        for (i in 1:length(r)) cosamt[[i]] <- cosamt[[i]][bdp >= 
            r[i], ]
        npoints <- sapply(cosamt, function(x) dim(x)[1])
    }
   
    if(is.expression(what)) midar <- sapply(cosamt, function(x) eval(what))
    if(is.function(what))   midar <- sapply(cosamt, what)
    
    result <- data.frame(r = r, midar = midar)
    #print(result)
    result <- fv(result, argu = "r", ylab = substitute(MIDAR(r), 
        NULL), valu = "midar", fmla = midar ~ r, alim = c(min(r), 
        max(r)), labl = c("r", "%s(r)"), desc = c("radius of circle", 
        "%s"), fname = "MIDAR")
    return(result)
}

# Modification of raoD function from package "picante" to accept a matrix of trait differences instead of ultrametric trees.
# We eliminate some operations that we do not need.
#The result by default is alpha diversity

raoDmod<-
function (comm, phy = NULL) 
{
    res <- list()
    if (is.null(phy)) {
        tij <- 1 - diag(x = rep(1, length(comm[1, ])))
    }
    else {
     
        #if(class(phy)!="phylo"){ # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! CAMBIADO 04/12/2019 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
       if(!inherits(phy, what="phylo")){
          # modificactions in the case a matrix enters instead of a tree
           # if(!is.matrix(phy) & class(phy)!="dist") stop("Phy must be a distance matrix") ## !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! CAMBIADO 04/12/2019 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	   if(!is.matrix(phy) &  !inherits(phy, what="dist")) stop("Phy must be a distance matrix") 

          if (is.matrix(phy)) phy <- as.dist(phy) #++
          dat <- match.comm.dist(comm, phy)#++
          comm <- dat$comm
          phy <- dat$dist #++
          tij <-as.matrix( phy/2) #++
         }
        #if(class(phy)=="phylo"){ ## !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! CAMBIADO 04/12/2019 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	if(inherits(phy, what="phylo")){
          if(!is.ultrametric(phy)) stop("Phylogeny must be ultrametric") 
          dat <- match.phylo.comm(phy, comm)
          comm <- dat$comm
          phy <- dat$phy
          tij <- cophenetic(phy)/2
         }# ++
     
      }
    x <- as.matrix(comm)
    S <- length(x[1, ])
    N <- length(x[, 1])
    total <- apply(x, 1, sum)
    samp.relabund <- total/sum(x)
    x.combined <- matrix(apply(x, 2, sum), nrow = 1)/sum(x)
    x <- sweep(x, 1, total, "/")
    D <- vector(length = N)
    names(D) <- rownames(x)
    for (k in 1:N) D[k] <- sum(tij * outer(as.vector(t(x[k, ])), 
        as.vector(t(x[k, ]))))
    res$Dkk <- D
    
   # All this is for obtaining matrix H (Among-community diversities excluding within-community diversity )
    # that we are not interested in.
   #
   #    Dkl <- matrix(nrow = N, ncol = N)
   #for (k in 1:N) {
    #    for (l in 1:N) {
     #       Dkl[k, l] <- sum(tij * outer(as.vector(t(x[k, ])), 
      #          as.vector(t(x[l, ]))))
       # }
    #}
    #row.names(Dkl) <- row.names(x)
    #colnames(Dkl) <- row.names(x)
    #H <- Dkl
    #res$Dkl <- Dkl
    #for (k in 1:N) {
    #    for (l in 1:N) {
    #        H[k, l] <- Dkl[k, l] - (Dkl[k, k] + Dkl[l, l])/2
    #    }
   # }
    #res$H <- H
    
    #res$total <- sum(tij * outer(as.vector(t(x.combined)), as.vector(t(x.combined))))
    
     res$alpha <- sum(res$Dkk * samp.relabund)
     
     #res$total <- sum(tij * outer(as.vector(t(x.combined)), as.vector(t(x.combined))), na.rm=TRUE)
     #res$beta <- res$total - res$alpha
     #res$Fst <- res$beta/res$total
     #res$tij<-tij
    return(res$alpha)
}

#####################################################################
# modificada a partir de "Kphylo.sp.envelope2"; Esta version es la que se llamaba  llamada "envelope.fv.idar4.R"

# toma como entrada el ppp original y como simulados solo los simulados de la especie focal (de simulador2: en data.frame)
# mippp: ppp multivariao
# mippp.sp.sim <- list of simulated focal patterns from simulador2.R
# mippp.sp <- observed univariate focal pp
# cross.idar: computing idar between different unities/age/size classes? IF TRUE, remove focal species from nmultivariate pp (mippp).
#      When computing "typical" isar, cross.idar=FALSE: the observed is replaced by the simulated
#      When computing cross-isar (btween different size-age-ckasses) the focal is not within the multivarite and 
#       there's nothin to replace and this is what happens when cross.idar=TRUE


# idar: name of the individual phylogenetic -area relationship to compute (isar, ipsear, ipsvar, ifdar, etc)
# tree: ape phylogenetic tree or variance/covarance matrix
# traits: data.frame of functional traits (species as columns, traits in rows)

# Es obligatorio suministar namesmark: mitable requiere un ppp multivariado con solo una columna de marcas y sin mnamesmark no se le puede suministrar

# fast envelope for isar analysis
# requires a list of simulated focal patterns


# En esta version 3, la matriz de distancias de gower se calcula SOLO UNA VEZ sobre toda la data.frame de traits
# Y se pasa a idar2 (en vez de traits) para ser subseteada segun sea necesario
# Eso hace que las distancias de gower entre especies sean siempre las mismas y no dependan de el tamano de la
# comunidad local (lo que ocurre ya que el Sijk usa el rango Rij para calcular las distancias individuales) 

envelope4idar <- function(mippp, mippp.sp.sim, mippp.sp, mimark=NULL, namesmark=NULL, r,
                                        idar="isar",
                                        buffer="adapt", bfw=NULL,nsim=NULL, nrank=1,
                                        tree = NULL,
                                        traits = NULL,
                                        cross.idar=FALSE,
                                        savefuns=TRUE,
                                       correct.phylo="exclude",
                                       correct.trait.na=FALSE,
                                       correct.trait="mean"
                                        ){


    if (!is.null(namesmark)) 
        mippp$marks <- factor((mippp$marks[namesmark][[1]]))
    if (!is.null(mimark)) 
        if (mimark %in% levels(mippp$marks) == FALSE) {
            stop(paste(mimark, " can't be recognized as a mark\n\n\n
                      have you indicated in which column of thedataframe are the species\n
                      marks? (argument 'namesmark'\n\n"))
        }
  
      # phylogenetic check and correction
      if(!is.null(tree)) tree<- checktree(tree=tree, mippp=mippp, idar=idar, correct.phylo=correct.phylo) ## NUEVA FUNCION de MANIPULACION DE TREES
      
      # Trait check and correction
       if (!is.null(traits)) traits <-checktraits(traits=traits, mippp=mippp, idar=idar, correct.trait.na=correct.trait.na, correct.trait=correct.trait)
       #if (!is.null(traits) & idar!="icwmar") traits <-checktraits(traits=traits, idar=idar, correct.trait.na=correct.trait.na, correct.trait=correct.trait)

   #*************************
   # BUFFER preprocessing
   # ************************* 
    #if the buffer is fixed, compute it only once and pass it to each simulation
    if (is.numeric(buffer) & is.null(bfw))  bfw <- owin(mippp$window$xrange + c(buffer, -buffer), 
                                                                        mippp$window$yrange + c(buffer, -buffer))

   #*************************
   # cross.idar preprocessing
   # *************************
    if (cross.idar == FALSE) {
        # we split the original ppp in focal (mippp.sp) and the rest (mippp)
        # in order to recompose  it with the simulated focals in idar2
        mippp.sp <- mippp[mippp$marks == mimark]
        # remove the focal species from the original mippp
        mippp <- mippp[mippp$marks != mimark]
    }

   #*************************
   # Computing observed and simulated idars
   # *************************
   # compute observed idar
    obs <- idar2(mippp.sp = mippp.sp, mippp = mippp, mimark = mimark, 
        buffer = buffer, bfw = bfw, r = r, cross.idar = cross.idar, 
        idar = idar, tree = tree, traits = traits)

    # compute simulated  idars   
    if (is.null(nsim)) nsim <- length(mippp.sp.sim)
    simvals = NULL
    cat("computing envelopes   ")
    for (i in 1:nsim) {
        progressreport(i, nsim)
        simvals <- cbind(simvals, idar2(mippp.sp = mippp.sp.sim[[i]], 
                          mippp = mippp, mimark = mimark, buffer = buffer, 
                          bfw = bfw, r = r, cross.idar = cross.idar, idar = idar, 
                         tree = tree, traits = traits))
    }
   #*************************
   # end of computing observed and simulated idars
   # *************************
   
    #*************************
    # Formating  and returning results
    #*************************
    # extract values for fv format
    if (nrank == 1) {
        lohi <- apply(simvals, 1, range)
    }
    else {
        lohi <- apply(simvals, 1, function(x, n) {
            sort(x)[n]
        }, n = c(nrank, nsim - nrank + 1))
    }
    lo <- lohi[1, ]
    hi <- lohi[2, ]
    lo.name <- paste("lower pointwise envelope of %s from simulations")
    hi.name <- paste("upper pointwise envelope of %s from simulations")
    m <- apply(simvals, 1, mean, na.rm = TRUE) # Beware that we ignore NA results !!!
      
    results <- data.frame(r = r, obs = obs, mmean = m, lo = lo, 
        hi = hi)

    if (idar == "isar") 
        ylab = substitute(ISAR(r), NULL)
    if (idar == "ipsrar") 
        ylab = substitute(IPSRAR(r), NULL)
    if (idar == "ipsvar") 
        ylab = substitute(IPSVAR(r), NULL)
    if (idar == "ipsear") 
        ylab = substitute(IPSEAR(r), NULL)
    if (idar == "ipscar") 
        ylab = substitute(IPSCAR(r), NULL)
    if (idar == "ifdar") 
        ylab = substitute(IFDAR(r), NULL)
    if (idar == "imntdar") 
        ylab = substitute(IMNTDAR(r), NULL)
    if (idar == "icwmar") 
        ylab = substitute(ICWMAR(r), NULL)
   if (idar == "icwmar.O") 
        ylab = substitute(ICWMAR.O(r), NULL)	
    if (idar == "iraodar")                                                
        ylab = substitute(IRaoDAR(r), NULL) 
 if (idar == "iraodar.O")                                                
        ylab = substitute(IRaoDAR.O(r), NULL) 

    result <- fv(results, argu = "r", ylab = ylab, valu = "obs", 
        fmla = . ~ r, alim = c(min(r), max(r)), labl = c("r", 
            "%s[obs](r)", "bar(%s)(r)", "%s[lo](r)", "%s[hi](r)"), 
        desc = c("distance argument r", "observed value of %s for data pattern", 
            "sample mean of %s from simulations", "lower pointwise envelope of %s from simulations", 
            "upper pointwise envelope of %s from simulations"), 
        fname = idar)
    class(result) <- c("envelope", "fv", "data.frame")
    attr(result, "dotnames") <- c("obs", "mmean", "hi", "lo")
    attr(result, "einfo") <- list(global = FALSE, nrank = nrank, 
        nsim = nsim, Nsim = nsim, dual = FALSE, nsim2 = nsim, 
        valname = "isar", VARIANCE = FALSE, csr = FALSE)
    if (savefuns == TRUE) 
        attr(result, "simfuns") <- data.frame(r = r, simvals)
    return(result)
}

#-------------------------------------------------------------------



####################################################################################################
####################################################################################################
## idar2
## derivada de  "Kphylo.sp2", para emplear en las envelopes. La entrada es mas flexible

idar2 <- function(mippp.sp, mippp, mimark, idar="isar", buffer,bfw, r, cross.idar=FALSE,  tree = NULL, traits = NULL) {
        
       #mippp.sp puede ser el univariado observado o viene de simulador2. contiene solo la especie focal (observada o simulada)
       # mippp tiene todas las otras especies excepto la focal.
       # viene de un paso externo (solo tiene que hacerse una vez)
       #    mippp <- mippp[mippp$marks!=mimark]
       # 
       # Si el bufer es fijo, tampoco hay que calcularlo cada vez: se suministra de un paso anterior:
       #    bfw<- owin( mippp$window$xrange + c(buffer,-buffer), mippp$window$yrange + c(buffer,-buffer))

    if (cross.idar == FALSE) {
       # We paste the focal ppp (observed or simulated) to the multivariate ppp    
        mippp$x <- c(mippp$x, mippp.sp$x)
        mippp$y <- c(mippp$y, mippp.sp$y)
        mippp$marks <- (c(as.character(mippp$marks), as.character(mippp.sp$marks)))
    }

    # If buffer is fixed, we trim now the focal pattern     
    if (!is.null(bfw)) 
        mippp.sp <- mippp.sp[inside.owin(mippp.sp, w = bfw)]

   # get local comunities around each focal tree for all  r's
    # result will be a list with length=  length(r) and each element in the list a data.frame with dimensions [mippp.sp$n, mippp$n]

    ######TODO: allow selection for functions mitable2, mitable3 etc  with sum of point sizes etc (from prototipomitable2m_3.R)
    cosamt <- mitable(mippp.sp, mippp, r) 
    
    # compute abundance in "rings" (for cwm.O computation)
    # BEWARE: this must be made before buffer correction
    # TODO: implement in frotran ???
    if( idar %in% c("iraodar.O", "icwmar.O")){
       cosamt.O <- cosamt
       for ( i in 2:length(cosamt.O)) cosamt.O[[i]] <- cosamt[[i]]-cosamt[[i-1]]
       # apply buffer correction if indicated
       if (buffer == "adapt") {
            bdp <- bdist.points(mippp.sp)
            for (i in 1:length(r))cosamt.O[[i]] <- cosamt.O[[i]][bdp >= r[i], ]
        }
    }
    
    
    #If indicated, correct edge effect with an adaptive buffer
    if(!idar %in% c("iraodar.O", "icwmar.O")){
       if (buffer == "adapt") {
           bdp <- bdist.points(mippp.sp)
          for (i in 1:length(r)) cosamt[[i]] <- cosamt[[i]][bdp >= r[i], ]
       }
    }

    #Define function to compute ISAR --------------------------------------------------------------------------------
    # TODO: this could be an external function, as psv, pse, FD, etc
    isar.r <- function(x) {
        # bivariate emptiness probability
        Ptj <- function(x) sum(x == 0)/length(x)
        #compute isar    
        result <- sum(apply(x, 2, function(x) 1 - Ptj(x)))
        return(result)
    }
    # ---- end function isar.r -------------------------------------------------------------------------------------------------------

   # Definition of function cwm.r
   # OJO: traits here is just a vector with the values of only one trait
   cwm.r<- function(x, traits){
            if(!is.null(dim(traits))) stop ("to compute icwmar, 'traits' mut be a vector with the values of juts one trait")
            return(sum(x*traits, na.rm=TRUE)/sum(x))
   }

    # Choose analysis
    if (idar == "icwmar") {
       traits.ok<- traits[names(traits)%in%colnames(cosamt[[1]])] # added 20/04/2016
       micwmar <-sapply(cosamt, function(tr) {
                         if (is.null(dim(tr)))   tr <- rbind(tr, tr) # If there is only one point, repeat its local assemblage to build a matrix and still be able to use "apply"
                         cwm.ind<-apply(tr, 1, function(x) cwm.r(x, traits=traits.ok))
                         cwm.ind[is.na(cwm.ind)] <- 0 # Set 0 fot those points without neighbours (i.e., no traits in their surroundings)
                        return(mean(cwm.ind))
            })
       result<-micwmar 
    }
    
     if (idar == "icwmar.O") {
           traits.ok<- traits[names(traits)%in%colnames(cosamt.O[[1]])] # added 20/04/2016
           micwmar <-sapply(cosamt.O, function(tr) { # use cosamt.O instead of cosamt
	                # If there is only one point, repeat its local assemblage 
	                #to build a matrix and still be able to use "apply"
                         if (is.null(dim(tr)))   tr <- rbind(tr, tr) 
                         cwm.ind<-apply(tr, 1, function(x) cwm.r(x, traits=traits.ok))
			 # Set 0 fot those points without neighbours (i.e., no traits in their surroundings)
                         cwm.ind[is.na(cwm.ind)] <- 0 
                        return(mean(cwm.ind))
            })
       result<-micwmar 
    }
    
    
    if (idar == "iraodar") {
       miraodar <-sapply(cosamt, function(x) {
             if (is.null(dim(x)))   x <- rbind(x, x)
	     if(dim(x)[1]<1) return(NA)
             res<- raoDmod(comm=x, phy=tree)
	     return(res)
           })
          result <- miraodar
    }
    
    if (idar == "iraodar.O") {
       miraodar <-sapply(cosamt.O, function(x) {
             if (is.null(dim(x)))   x <- rbind(x, x)
	     if(dim(x)[1]<1) return(NA)
             res<- raoDmod(comm=x, phy=tree)
	     return(res)
           })
          result <- miraodar
    }
       

    if (idar == "ifdar") {
       #Compute IFDAR(r)  (i.e., IFdisp) for each radius r
        mifdar <- sapply(cosamt, function(x) {
                       if (is.null(dim(x)))   x <- rbind(x, x)
                       res <- fdis(x, traits = traits) 
                       return(res)
        })
        result <- mifdar
    }
    
   if (idar == "isar") {
        #Compute ISAR(r) for each radius r
        misar <- sapply(cosamt, function(x) {
                      if (is.null(dim(x)))  x <- rbind(x, x)
                      res <- isar.r(x)
                      return(res)
        })
        result <- misar
    }

    if (idar == "ipsear") {
       # first for each individual
        mipsear <- lapply(cosamt, function(x) {
            if (is.null(dim(x)))  x <- rbind(x, x)
            if (sum(colSums(x) > 0) > 1)   res <- pse(x, tree = tree)$PSE
            if (sum(colSums(x) > 0) <= 1)  res <- rep(NA, dim(x)[1]) # repit NA for all individuals of he focal species
            return(res)
        })
        # then average ipse for each radius
        result <- sapply(mipsear, mean, na.rm = T)# TODO: optional na.rm. At short distances lot of NA's and maybe unreliable means
    }

    if (idar == "ipsvar") {
        # first for each individual
        mipsvar <- lapply(cosamt, function(x) {
            if (is.null(dim(x)))  x <- rbind(x, x)
            if (sum(colSums(x) > 0) > 1)   res <- psv(x, tree = tree, compute.var = F)$PSVs
            if (sum(colSums(x) > 0) <= 1) res <- rep(NA, dim(x)[1]) # repit NA for all individuals of he focal species
            return(res)
        })
        # then average ipsv for each radius
        result <- sapply(mipsvar, mean, na.rm = T)# TODO: optional na.rm. At short distances lot of NA's and maybe unreliable means
    }

    if (idar == "ipsrar") {
         # first for each individual
        mipsrar <- lapply(cosamt, function(x) {
            if (is.null(dim(x)))   x <- rbind(x, x)
            if (sum(colSums(x) > 0) > 1)   res <- psr(x, tree = tree, compute.var = F)$PSR
            if (sum(colSums(x) > 0) <= 1)  res <- rep(NA, dim(x)[1]) # repit NA for all individuals of he focal species
            return(res)
        })
        # then average ipsr for each radius
        result <- sapply(mipsrar, mean, na.rm = T)# TODO: optional na.rm. At short distances lot of NA's and maybe unreliable means
    }

    if (idar == "ipscar") {
        # first for each individual
        mipscar <- lapply(cosamt, function(x) {
            if (is.null(dim(x))) 
                x <- rbind(x, x)
            if (sum(colSums(x) > 0) > 1) 
                res <- psc(x, tree = tree)$PSCs
            if (sum(colSums(x) > 0) <= 1) 
                res <- rep(NA, dim(x)[1])
            return(res)
        })
        # then average ipsc for each radius
        result <- sapply(mipscar, mean, na.rm = T)# TODO: optional na.rm. At short distances lot of NA's and maybe unreliable means
    }

    if (idar == "imntdar") {
        # first for each individual
        mimntdar <- lapply(cosamt, function(x) {
            if (is.null(dim(x))) 
                x <- rbind(x, x)
            if (sum(colSums(x) > 0) > 1) 
                res <- mntd(x, dis = tree, abundance.weighted = FALSE)
            if (sum(colSums(x) > 0) <= 1) 
                res <- rep(NA, dim(x)[1])
            return(res)
        })
        # then average imntdar for each radius
        result <- sapply(mimntdar, mean, na.rm = T) # TODO: optional na.rm. At short distances lot of NA's and maybe unreliable means
    }
    return(result)
}


######################################################
# Funcion simulador2
# para generar heterogeneos poisson para calcular las envueltas
#
# la salida no es un ppp sino un dataframe con la especie
# simulada. Esto evita tener que escribir repetidamente 
# cientos de miles de datos que no se simulan
# (se simulan como mucho cientos de individuos de la especie focal)
######################################################

simulador2 <- function(mimark, milambda, nsim=99){
    cosapp <- vector("list", nsim)
    cosapp <- lapply(cosapp, function(x) {
                      x<- rpoispp(milambda)
                      x$marks<-factor(rep(mimark, x$n))
                      return(x)
                      }
              )
return(cosapp)
}
#-------------------------------------------------------------------

###############################################################################
#
#    IPSEAR.R
#
#   Individual Phylogenetic Species Evennes -Area Relationship
#
#  $Version: 0.1 $  $Date: 2013/02/17 23:48:03 $
#
#   function to compute Indivdual Phylogenetic Species Evennes Area Relationships (i.e., acumulation of PSE curves
#    around individual species) in a community.
#
#    Value: an object of class fv , bassicaly a data.frame with the following
#    elemnts:
#     r: vector of radii at which IPSEAR(r) has been estimated
#     ipsv: vector with the IPSEAR(r) values
#     ###npoints: number of focal points employed to estimate isar(r) at each 
#              radius r
#
#    Arguments:
#     mippp:    Multivariate point pattern (ppp of spatstat)
#     mippp.sp: Focal (unmarked) point pattern (ppp of spatstat) or character
#                string indicating one of the marks in the multivariate mippp 
#     mimark:   character string indicating one of the marks in the
#                multivariate mippp
#     namesark: name of the column with species names in the data.frame of
#                marks (only for multimarked ppp's)
#     tree:    A phylogenetic tree in "phylo" format (ape) or phylogenetic covariance matrix 
#     r:        vector with the sequence of radii (>0) at which estimate
#                ISAR(r)
#     buffer:   a number indicating the width around the window of the focal 
#                point pattern that will be excluded from the computations of
#                ISAR(r) to control de edge effect, or the string "adapt", 
#                indicating that an adaptive border (of width ri) will be 
#                excluded for the computation of each ISAR(ri)
#     bfw:      an owin object indicating a subset of the focal point pattern 
#                that will be employed to compute ISAR(r)
#
#   Details:
#     btw can be employed to select different habitats and compute ISAR((R) i 
#      each habitat

 




 
ipsear<- function(mippp, mippp.sp=NULL, mimark=NULL,  namesmark=NULL, tree=NULL,
                      r=NULL, buffer=0, bfw=NULL, correct.phylo="mean") {

  # Check mippp
  #If species marks are within a data.frame of marks, get this column and 
  # discard the rest
  if(!is.null(namesmark)) mippp$marks <- factor((mippp$marks[namesmark][[1]]))  # OJO: que representa el [[1]] del final: es para coger los valores. si no se pone da problemas !!!!!!!!!!!!!!!!!!!!!!!!!!!
  
    # Check tree  
    # this is for checktree
    idar <-"ipsear"
    if(is.null(tree)) stop("you should provide a phylogenetic tree to compute ipscar")
    #if(class(tree)=="phylo") tree<- vcv.phylo(tree, corr = TRUE) #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! CAMBIADO 04/12/2019 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    if(inherits(tree, what="phylo"))  tree<- vcv.phylo(tree, corr = TRUE)
    if(!is.matrix(tree)) stop("you should provide a phylogenetic tree or a phylogenetic covariance matrix to compute ipscar")
    tree <- checktree(tree=tree,  mippp=mippp, idar=idar, correct.phylo=correct.phylo)
  
  
  # Check mippp.sp
  #If mippp.sp has not been provided as a ppp, build it from the multivariate
  # mippp and the provided mark
  if(!is.null(mippp.sp) & !is.ppp(mippp.sp)) {
   mippp.sp <- NULL
   mimark <- mippp.sp
  }

  if(!is.null(mimark)) if(mimark%in%levels(mippp$marks)==FALSE){
     stop(paste(mimark, " can't be recognized as a mark\n\n
     have you indicated in which column of thedataframe are the species
      marks? (argument 'namesmark'\n\n"))
      }
  if (is.null(mippp.sp)) mippp.sp<- mippp[mippp$marks==mimark]
       
  #If indicated, correct edge effect with a fixed buffer of size "buffer"
  # or with the window "bfw"
  if(buffer!="adapt"){	
    if(is.null(bfw)) bfw<- erosion( mippp$window, buffer)
    mippp.sp <- mippp.sp[inside.owin(mippp.sp, w=bfw)]
    npoints <- rep(mippp.sp$n, length(r))
    names(npoints) <- r
  }
       

  #Tabulate the number of individuals of each species (i.e. types) around each 
  # individual of the focal species within circles of each of the r radii
  
  cosamt <- mitable(mippp.sp, mippp, r)
  
   # Check that there are enough differnt individuasl to compute psv (and avoid error!). Frequent problem for short r's
    cosamt.ok<- sapply(cosamt, function(y) any(apply(y,1,function(x) sum(x>0))>1)) #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 17/02/2015
    
	
  #If indicated correct edge effect with an adaptive buffer
  if(buffer=="adapt"){
    bdp <- bdist.points(mippp.sp)
    for (i in 1:length(r)) cosamt[[i]]<-cosamt[[i]][bdp>=r[i],]
    npoints <- sapply(cosamt, function(x) dim(x)[1])
  }
  	
   
  #Compute IPSVAR(r) for each radius r
   # preset result to NA
    ipsear<- rep(NA, length(r))
  
  # first for each individual  (for those individuals which have enough neighbors)
      #ipsear <- lapply(cosamt, function(x) pse(x, tree=tree)$PSE)
   ipsear[cosamt.ok] <- lapply(cosamt[cosamt.ok], function(x) pse(x, tree = tree)$PSE)
  
  
  # then average ipsv for each radius
  ipsear <- sapply(ipsear, mean, na.rm=T) # TODO: optional na.rm. At short distances lot of NA's and maybe unreliable means

  # return results
  
   result <- data.frame(r=r, ipsear=ipsear)
  
   result <- fv(result, argu="r", ylab=substitute(IPSEAR(r), NULL),
              valu="ipsear", fmla=ipsear ~ r,
              alim=c(min(r),max(r)),
              labl=c("r", "%s(r)"),
              desc=c("radius of circle",
                     "%s"),
              fname="IPSEAR")

  
  
  
  #class(result)<-c("isar", class(result))
  return(result)
}
###############################################################################

  #*************************
     # Phylogenetic manipulations  TODO: phylogenetic manipulations as an external function
     #************************* 
    # tree : a phylo object (ape) or a phylogenetic covariance matrix

checktree<- function(tree,  mippp, idar, correct.phylo){
   
        treeold <- tree
        #if (class(tree) == "phylo")  tree <- vcv.phylo(tree, corr = TRUE) #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! CHANGED 04/12/2019!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	if (inherits(tree, what= "phylo")) tree <- vcv.phylo(tree, corr = TRUE)
        sp.nok <- which(is.na(match(levels(mippp$marks), dimnames(tree)[[1]])))
        len.sp.nok <- length(sp.nok)
        names.sp.nok <- levels(mippp$marks)[sp.nok]
        # if there is some species missing in the phylogenetic tree
        if (len.sp.nok > 0) {
            # option 1) include missing species in the tree with a constant mean phylogenetic covariance
            if (correct.phylo == "mean") {
                meanphylo <- mean(as.dist(tree))
                dimtree <- dim(tree)
                newtree <- matrix(1, nrow = dimtree[1] + len.sp.nok, 
                  ncol = dimtree[1] + len.sp.nok)
                newtree[1:len.sp.nok, ] <- meanphylo
                newtree[, 1:len.sp.nok] <- meanphylo
                newtree[-(1:len.sp.nok), -(1:len.sp.nok)] <- tree
                diag(newtree) <- 1
                dimnames(newtree) <- lapply(dimnames(tree), function(x) c(names.sp.nok, 
                  x))
                warning(paste("species", names.sp.nok, "was not in the phylotree and has been included with constant mean phylogenetc covariance\n"))
                tree <- newtree
            }
            # option 2) eliminate missing species from the ppp
            if (correct.phylo == "exclude") {
                numbers.nok <- table(mippp[mippp$marks %in% names.sp.nok]$marks)[names.sp.nok]
                mippp <- mippp[!mippp$marks %in% names.sp.nok]
                mippp$marks <- factor(mippp$marks)
                warning(paste("species ", names.sp.nok, ", with ", 
                  numbers.nok, " individuals, was not in the phylotree and has been excluded from the analysis\n", 
                  sep = ""))
            }
        }
        if (idar == "imntdar") { # OJO: poner al principio de la funcion, para que corrija las especies faltantes (pasar cophenetic a "matrix" y al final de todas las correcciones devolver otr vez como "as.dist"
            #if (class(treeold) != "phylo")  #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! CHANGED 04/12/2019!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	    if (!inherits(treeold, what= "phylo") )   stop("Computing IMNTDAR requires a phylogenetic tree of class 'phylo'\n\n")
            tree <- cophenetic(treeold)
        }
    return(tree)
    }
         

      #*************************
      # Trait manipulations. TODO: trait manipulations as an external function
      # ************************* 

checktraits<- function(traits, mippp, idar, correct.trait.na, correct.trait){

        # If traits come as a vector , do this:
        #if (is.null(dim(traits)) & class(traits) != "dist") {   #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! CHANGED 04/12/2019!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	if (is.null(dim(traits)) & !inherits(traits, what= "dist")) {
            # first, check that the vector of traits has names simuilar to species names in ppp$marks 
            if (sum(!is.na(match(levels(mippp$marks), names(traits)[[1]]))) < 1)   stop(
            "the vector of traits should be named and should wear the same labels as the names\n of species in ppp$marks")

            # then deal with NA's in the vector of traits
            meantrait <- mean(traits, na.rm = T)
            if (correct.trait.na == TRUE) {
                traits[is.na(traits)] <- meantrait
            }

            # check that all species in the community are in the vector of traits
            sp.nok <- which(is.na(match(levels(mippp$marks), 
                                    names(traits))))
            len.sp.nok <- length(sp.nok)
            names.sp.nok <- levels(mippp$marks)[sp.nok]

            # if there is some species missing in the vector of traits...
            if (len.sp.nok > 0) {

                # option 1) include missing species in the vector with a mean trait value
                if (correct.trait == "mean") {
                  # repeat mean traits as many times as there are missing species
                  traitplus <- NULL
                  for (i in 1:len.sp.nok) traitplus <- c(traitplus, meantrait)
                  names(traitplus) <- names.sp.nok
                  traits <- c(traits, traitplus)
                  traits <- traits[order(names(traits))]
                  warning(paste("species", names.sp.nok,
                  "was not in the vector of traits and\n has been included with mean trait value\n"))
                }

                # option 2) eliminate missing species from the ppp
                if (correct.trait == "exclude") {
                  numbers.nok <- table(mippp[mippp$marks %in% names.sp.nok]$marks)[names.sp.nok]
                  mippp <- mippp[!mippp$marks %in% names.sp.nok]
                  mippp$marks <- factor(mippp$marks)
                  warning(paste("species ", names.sp.nok, ", with ", numbers.nok,
                  " individuals, was not in the data.frame of traits and has\n been excluded from the analysis\n",
                    sep = ""))
                }
            }
            #traits se sustituye por la matriz de distancias de traits solo si vamos a calcular el IFDAR
            if(idar=="ifdar") traits <- as.matrix(gowdis(traits)) 
        }

        # If traits come as a data.frame, do this:
        if (!is.null(dim(traits))) {# for traits as a data.frame
            # first, check that the data.frame of traits has dimnames simuilar to species names in ppp$marks 
            if (sum(!is.na(match(levels(mippp$marks), dimnames(traits)[[1]]))) < 1)  stop(
            "the rownames of the data.frame of traits should  wear the same labels as the names\n of species in ppp$marks")
            # then deal with NA's in the table of traits
            meantrait <- apply(traits, 2, mean, na.rm = T)
            if (correct.trait.na == TRUE) {
                for (i in 1:dim(traits)[2]) traits[is.na(traits[, i]), i] <- meantrait[i]
            }

            # check that all species in the community are in the table of traits
            sp.nok <- which(is.na(match(levels(mippp$marks), dimnames(traits)[[1]])))
            len.sp.nok <- length(sp.nok)
            names.sp.nok <- levels(mippp$marks)[sp.nok]

            # if there is some species missing in the table of traits
            if (len.sp.nok > 0) {

                # option 1) include missing species in the table with a mean trait value
                if (correct.trait == "mean") {
                  # repeat mean traits as many times as there are missing species
                  traitplus <- NULL
                  for (i in 1:len.sp.nok) traitplus <- rbind(traitplus,  meantrait)
                  row.names(traitplus) <- names.sp.nok
                  traits <- rbind(traits, traitplus)
                  traits <- traits[order(rownames(traits)), , drop = FALSE]
                  warning(paste("species", names.sp.nok,
                  "was not in the data.frame of traits\n and has been included with mean trait value\n"))
                }

                # option 2) eliminate missing species from the ppp
                if (correct.trait == "exclude") {
                  numbers.nok <- table(mippp[mippp$marks %in% names.sp.nok]$marks)[names.sp.nok]
                  mippp <- mippp[!mippp$marks %in% names.sp.nok]
                  mippp$marks <- factor(mippp$marks)
                  warning(paste("species ", names.sp.nok, ", with ", numbers.nok, 
                   " individuals, was not in the data.frame of traits and has been\n excluded from the analysis\n",
                     sep = ""))
                }
            }
            #traits se sustituye por la matriz de distancias de traits solo si vamos a calcular el IFDAR
            if(idar=="ifdar") traits <- as.matrix(gowdis(traits)) 
        }
        # if traits comes as a distance semimatrix (dist) transform it to a matrix
         # if (class(traits) == "dist")  traits <- as.matrix(traits) #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! CHANGED 04/12/2019!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	 if (inherits(traits, what= "dist"))  traits <- as.matrix(traits)
        
       #if traits come as a matrix of traits instead of a data.frame and we are computing ifdar, do this:
        if (is.matrix(traits) & idar=="ifdar"){ 
             if (dim(traits)[1] != dim(traits)[2])  traits <- as.matrix(gowdis(traits))
         }
   return(traits) 
   }
   

LF.gof<- function(X, rmin=NULL, rmax=NULL, na.rm=TRUE){
    
   # controls that is using an envelope object and that it has been computed with the argument savefuns=TRUE
    if(sum(!is.na(match(class(X),"envelope")))==0) stop("LF.gof only accept spatstat envelope objects")
    if(is.null(attributes(X)$simfuns)) stop ("did you compute the envelopes with the argument 'savefuns=TRUE'?") 
    
    # get data
    r <- as.data.frame(attributes(X)$simfuns)[,1]
    H1 <- X$obs
    Hi <- as.data.frame(attributes(X)$simfuns)[,-1]
    
    #Join observed and simulated functions
    H <- cbind(H1,Hi) 
    
    
    # select the range of integrable results
    if (!is.null(rmin) & !is.null(rmax)){ H <- H[r >= rmin & r <= rmax,]}
    if (!is.null(rmin) & is.null(rmax)) { H <- H[r >= rmin,]}
    if (is.null(rmin) & !is.null(rmax)) { H <- H[r <= rmax,]}
    
    # count how may NA values for each distance r
    na.count<-apply(H,1, function(x) sum(is.na(x)))
    na.count.by.r <-na.count[na.count>0] 

    s <- dim(H)[2]

    # compute summary statistics for the observed (i.e., first) pattern 
    Hmeans <- rowSums (H[,-1],  na.rm = na.rm)/(s-1)
    u <- sum((H[,1]-Hmeans)^2, na.rm=T)
     
     # compute summary statistics for the simulated patterns
     for ( i in 2:s) {
         Himeans <- rowSums (H[,-i],na.rm=na.rm)/(s-1) 
         u <- c(u, sum((H[,i]-Himeans)^2, na.rm=na.rm)) 
       }
     p <-  1-((rank(u)[1]-1)/(s))
     
return(list(u=u[1], p=p,na.count.by.r=na.count.by.r))
}

#Fucnion para simular patron individual de una especie focal en un patron multivariado, dejando el resto de especies en sus coordenadas
# pp: patron multivariado
# mimark: nombre de la especie (marca) que se quiere simular
# simulate: lista de patrones individuales simulados o expresion para simular el patron individual
#nsim: numero de simulaciones
# nmin: numero minimo de individuos esperados en el patron simulado.

multifocalsimulator <- function(pp, mimark, simulate,nsim=99,nmin=NULL){
    lista.sim<-list()
    pp.sp<-split(pp)
    if(is.list(simulate)){
       for (i in 1:nsim){
       progressreport(i,nsim)
         pp.sp[[mimark]]<-simulate[[i]] 
         split(pp)<-pp.sp
	 lista.sim[[i]]<- pp
	 }
    }
    if(is.expression(simulate)){
       for (i in 1:nsim){
       progressreport(i,nsim)
         pp.sp[[mimark]]<-eval(simulate) 
	 if(!is.null(nmin)){ # control para que los simulados no tengan menos de "nmin" puntos
	    while(pp.sp[[mimark]]$n<nmin){pp.sp[[mimark]]<-eval(simulate) }
	    }
         split(pp)<-pp.sp
	 lista.sim[[i]]<- pp
	 }
    }
    
    return(lista.sim)
}

# Function fdisp, copied from package FD 1.0-12.3
fdisp <-
function (d, a, tol = 1e-07) 
{
    if (!inherits(d, "dist")) 
        stop("'d' must be a 'dist' object.")
    n <- attr(d, "Size")
    if (is.null(attr(d, "Labels"))) 
        stop("'d' must have labels.", "\n")
    else sn.d <- attr(d, "Labels")
    if (missing(a)) {
        ab.names <- list("Community1", sn.d)
        a <- matrix(1, 1, n, dimnames = ab.names)
    }
    com <- nrow(a)
    if (!is.matrix(a)) 
        stop("'a' must be a matrix.")
    if (ncol(a) != n) 
        stop("Number of columns in 'a' must be equal to the number of objects in 'd'.")
    if (is.null(colnames(a))) 
        stop("'a' must have column names", "\n")
    else sn.a <- colnames(a)
    if (any(sn.d != sn.a)) 
        stop("Species labels in 'd' and 'a' need to be identical and ordered alphabetically (or simply in the same order).", 
            "\n")
    a[which(is.na(a))] <- 0
    abun.sum <- apply(a, 1, sum)
    if (any(abun.sum == 0)) 
        stop("At least one community has zero-sum abundances (no species).", 
            "\n")
    abun.sum2 <- apply(a, 2, sum)
    if (any(abun.sum2 == 0)) 
        stop("At least one species does not occur in any community (zero total abundance across all communities).", 
            "\n")
    if (any(is.na(d))) 
        stop("NA's in the distance matrix.", "\n")
    A <- matrix(0, ncol = n, nrow = n)
    A[row(A) > col(A)] <- -0.5 * d^2
    A <- A + t(A)
    G <- bicenter.wt(A)
    e <- eigen(G, symmetric = TRUE)
    vectors <- e$vectors
    eig <- e$values
    w0 <- eig[n]/eig[1]
    if (w0 > -tol) 
        r <- sum(eig > (eig[1] * tol))
    else r <- length(eig)
    vectors <- vectors[, 1:r, drop = FALSE] %*% diag(sqrt(abs(eig <- eig[1:r])), 
        r)
    dimnames(vectors) <- list(colnames(a), NULL)
    pos <- eig > 0
    avg.dist.cent <- rep(NA, nrow(a))
    names(avg.dist.cent) <- row.names(a)
    for (i in 1:com) {
        pres <- which(a[i, ] > 0)
        nb.sp <- nrow((unique(vec <- vectors[pres, , drop = F])))
        if (nb.sp >= 2) {
            w <- a[i, pres]
            centroid <- apply(vec, 2, weighted.mean, w = w)
            dist.pos <- sweep(vec[, pos, drop = F], 2, centroid[pos])
            dist.pos <- rowSums(dist.pos^2)
            if (any(!pos)) {
                dist.neg <- sweep(vec[, !pos, drop = F], 2, centroid[!pos])
                dist.neg <- rowSums(dist.neg^2)
            }
            else dist.neg <- 0
            zij <- sqrt(abs(dist.pos - dist.neg))
            avg.dist.cent[i] <- weighted.mean(zij, w)
        }
        else avg.dist.cent[i] <- 0
    }
    return(list(FDis = avg.dist.cent, eig = eig, vectors = vectors))
}

c------------------------------------------------------------------- 
      SUBROUTINE mtb(x1,y1,x2,y2,l1,l2,nombresp,nsp,r,nr,ltab,
     +                    abu)
      integer l1,l2,nombresp(l2),nsp,nr,ltab
      double precision x1(l1),y1(l1),x2(l2),y2(l2), r(nr),
     +                 abu(ltab*nr)

      integer i,j,nabu, rabu !,n
      double precision zero,dist  !(l1*l2)
      zero=0.0
      DO i= 1,l1
         DO j= 1,l2
c            n=n+1
            nabu =  nsp*(i-1) + nombresp(j)    ! identifica la especie en la que anotar la cuenta       
c            dist(n) = sqrt((((x1(i)-x2(j))**2)+((y1(i)-y2(j))**2)))
            dist = sqrt((((x1(i)-x2(j))**2)+((y1(i)-y2(j))**2)))
            DO k=1,nr
c               IF(dist(n).le.r(k)) THEN
               IF(dist.gt.zero.and.dist.le.r(k)) THEN
                 rabu= (l1*nsp*(k-1))+nabu   ! para la especie identificada, busca la distancia en la que anotar la cuenta
                 abu(rabu) = abu(rabu) + 1
               END IF
            END DO   
         END DO
         
         
      END DO
      
      RETURN
      END

c-------------------------------------------------------------------       
c     mtb; renombrado el nombre de la subrutina para que no conflicte 
c          con el nombre de la función en R
c     mitable: renombrada sin numero para incluir en el paquete ISAR
c       es idéntca a mitable4
c     mitable4: se diferencia de mitable3 en que incorpora en el IF
c               que "dist.gt.0", para no contar el propio arbol focal
c     mitable3: se diferencia de mitable2 en que "dist" es interna
c      al programa fortran y no se devuelve a R, con lo que en teoría
c      se debería aumantar rapidez y limmitar problemas de memoria
c           
c     mitable: calcula vecinos de cada especie  del patron 2 
c     [coordenadas (x2, y2), con l2 individuos],dentro del círculo
c     de radio r  alrededor de cada individuo del patron 1
c     [coordenadas(x1, y1), con l1 individuos]
c     Otras variables suministradas:
c     nsp: numero de especies diferentes en el patron 2.Es un integer
c            
c     nombresp: vector de integers con los numeros que representan
c     a cada especie del patron2.se obtiene con
c      unique(as.numeric(factor(marks)))
c     dist: vector de distancias que se quiere calcular
c     abu: vector de abundancias de cada especie como vecinas de 
c     cada punto del patron 1. Es la representación vectorizada de
c     la tabla de "comunidades".
 

controldis <- function(d, m, mimark) {
         
	 #function to control and the distance object and
	# to reasure that it match the species in the ppp
	
       # TODO: control that d is a named/unnamed vector

        # meter opcion para que saque la media de los NA en el vector final.
	# (deberia ser un argumento tambien de pisar y de risar)

        
      if(is.null(d)){
	      d<- rep(1, ncol(m))
	      return(d)
      }
   
     #if(class(d)=="dist"){  # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!   CAMBIADO 04/12/2019 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     if(inherits (d, what="dist")){ 
         d<- as.matrix(d)
     }
      
      if(is.matrix(d)){
         cuales <- colnames(d)%in%colnames(m)
	 if(sum(cuales)!=ncol(m)) stop("Names of species in the ppp and in the 'd'-distance matrix dont match")
	 if(!mimark%in%rownames(d)) stop("Name of the focal species not found  in the 'd'-distance matrix")
		 
	 # extract the distances to focal species	 
	 d<- d[mimark, cuales]
	 #match names ofspecies in cosamt and in the distance vector and re-order
	 names.ok<-match (colnames(m),names(d))
	 d <- d[names.ok]
       }
  
        return(d)
}

#---------------------------------------------------------------------------------------------------





pisar<-
function (mippp, mippp.sp = NULL, mimark = NULL, namesmark = NULL, d=NULL, 
    r = NULL, buffer = 0, bfw = NULL) 
{
    
    if (!is.null(namesmark)) 
        mippp$marks <- factor((mippp$marks[namesmark][[1]]))
    if (!is.null(mippp.sp) & !is.ppp(mippp.sp)) {
        mippp.sp <- NULL
        mimark <- mippp.sp
    }
    if (!is.null(mimark)) 
        if (mimark %in% levels(mippp$marks) == FALSE) {
            stop(paste(mimark, " can't be recognized as a mark\n\n\n     have you indicated in which column of thedataframe are the species\n      marks? (argument 'namesmark'\n\n"))
        }
    if (is.null(mippp.sp)) 
        mippp.sp <- mippp[mippp$marks == mimark]
    if (buffer != "adapt") {
        if (is.null(bfw)) 
            bfw <- erosion(mippp$window, buffer)
        mippp.sp <- mippp.sp[inside.owin(mippp.sp, w = bfw)]
        npoints <- rep(mippp.sp$n, length(r))
        names(npoints) <- r
    }
    cosamt <- mitable(mippp.sp, mippp, r)
    if (buffer == "adapt") {
        bdp <- bdist.points(mippp.sp)
        for (i in 1:length(r)) cosamt[[i]] <- cosamt[[i]][bdp >= 
            r[i], ]
        npoints <- sapply(cosamt, function(x) dim(x)[1])
    }
    
    #--------------------------------------------------------------------
    # vector de distancias de la focal al resto.
      d<- controldis(d, cosamt[[1]], mimark)
    #--------------------------------------------------------------------
    
    Ptj <- function(x) sum(x == 0)/length(x)
    pisar.r <- function(x) {
        result <- sum(d* apply(x, 2, function(x) 1 - Ptj(x))) # cambio:multiplicamos por el vector d
        return(result)
    }
    pisar <- sapply(cosamt, pisar.r)
    result <- data.frame(r = r, pisar = pisar)
    result <- fv(result, argu = "r", ylab = substitute(PISAR(r), 
        NULL), valu = "pisar", fmla = pisar ~ r, alim = c(min(r), 
        max(r)), labl = c("r", "%s(r)"), desc = c("radius of circle", 
        "%s"), fname = "PISAR")
    return(result)
}

#-----------------------------------------------------------------------------------------------------------------------------------------------

risar<- 
function (mippp, mippp.sp = NULL, mimark = NULL, namesmark = NULL, d=NULL, d0=NULL,
    r = NULL, buffer = 0, bfw = NULL) 
{
    
    if (!is.null(namesmark)) 
        mippp$marks <- factor((mippp$marks[namesmark][[1]]))
    if (!is.null(mippp.sp) & !is.ppp(mippp.sp)) {
        mippp.sp <- NULL
        mimark <- mippp.sp
    }
    if (!is.null(mimark)) 
        if (mimark %in% levels(mippp$marks) == FALSE) {
            stop(paste(mimark, " can't be recognized as a mark\n\n\n     have you indicated in which column of thedataframe are the species\n      marks? (argument 'namesmark'\n\n"))
        }
    if (is.null(mippp.sp)) 
        mippp.sp <- mippp[mippp$marks == mimark]
    if (buffer != "adapt") {
        if (is.null(bfw)) 
            bfw <- erosion(mippp$window, buffer)
        mippp.sp <- mippp.sp[inside.owin(mippp.sp, w = bfw)]
        npoints <- rep(mippp.sp$n, length(r))
        names(npoints) <- r
    }
    cosamt <- mitable(mippp.sp, mippp, r)
    if (buffer == "adapt") {
        bdp <- bdist.points(mippp.sp)
        for (i in 1:length(r)) cosamt[[i]] <- cosamt[[i]][bdp >= 
            r[i], ]
        npoints <- sapply(cosamt, function(x) dim(x)[1])
    }
    
    #--------------------------------------------------------------------
    # vector de distancias de la focal al resto.
    # La entrada "d" puede ser una matriz, una dist o un vector.
    # en todos los casos con "names" identicos (y al final en el mismo orden) que los de cosamt 
    # que salen de los levels de las marcas delppp
    # la salida para calcular risar es un vector de longitud igual al numero de especies (columnas) en cosamt
    
     
     d<- controldis(d=d, m=cosamt[[1]], mimark)
     d0<- controldis(d=d0, m=cosamt[[1]], mimark)
     
    
    #--------------------------------------------------------------------
    
    Ptj <- function(x) sum(x == 0)/length(x)
    pisar.r <- function(x) {
        result <- sum(d* apply(x, 2, function(x) 1 - Ptj(x))) # cambio:multiplicamos por el vector d
        return(result)
    }
    isar.r <- function(x) {
        result <- sum(d0* apply(x, 2, function(x) 1 - Ptj(x))) # cambio:multiplicamos por el vector d
        return(result)
    }
    pisar <- sapply(cosamt, pisar.r)
    isar <- sapply(cosamt, isar.r)
    risar<-pisar/isar
    result <- data.frame(r = r, risar = risar)
    result <- fv(result, argu = "r", ylab = substitute(rISAR(r), 
        NULL), valu = "risar", fmla = risar ~ r, alim = c(min(r), 
        max(r)), labl = c("r", "%s(r)"), desc = c("radius of circle", 
        "%s"), fname = "rISAR")
    return(result)
}

##############################################################################
#
#   mitable.R
#   function to tabulate number of (occurence of) different species (i.e.
#    types) of a multivariate point pattern ppp2 within circles of radius r
#    around each individuals of a (unmarked) point pattern ppp1
#
#
#    Value: a list of length r; each element of the list a data.frame with
#            nrows = number of points in ppp1 and ncols = unique(ppp2$marks),
#            each cell being the counts of each type of pp2 points within a
#            radius r of the corresponding point of ppp1
#    Arguments:
#     ppp1: Focal (unmarked) point pattern (ppp of spatstat) or data.frame
#           with columns named "x" an "y"
#     ppp2: Multivariate point pattern (ppp of spatstat) or data.frame whith 
#           columns named "x", "y" and "marks"
#     r:    vector with the sequence of radii (>0) at which tabulate counts
#
#



mitable <- function(ppp1,ppp2,r){
    
    if(is.null(ppp2$marks)) stop("ppp2 should be a multivariate ppp or a data.frame with a '$marks' column")
    
    x1 <- ppp1$x
    y1 <- ppp1$y
    
    x2 <- ppp2$x
    y2 <- ppp2$y
    
    l1<- length(x1)
    l2<- length(x2)

    sp<- factor(ppp2$marks)
    especies <-levels (sp)

    nombresp <-  as.numeric(sp)
    nomsp <- unique(nombresp)
    nsp <-length(nomsp)
    nr <-length(r)
    ltab<-l1*nsp #length of a the matrix ppp1*nsp
 
    
    # mitable.f renamed mitablee.f to avoid troubles with registrations etc 27.04.2017
    ans <- .Fortran('mtb', x1=as.double(x1),y1=as.double(y1),
                    x2=as.double(x2),y2=as.double(y2),
                    l1=as.integer(l1), l2=as.integer(l2),
                    nombresp=as.integer(nombresp), nsp=as.integer(nsp),
                    r=as.double(r), nr=as.integer(nr),ltab=as.integer(ltab),
                    abu=double(ltab*nr),
                    PACKAGE="idar")
    
       
      
      result=vector(mode="list", length=nr)
      names(result)=r
      for(i in 1:nr){
        result[[i]]<-matrix(ans$abu[(((i-1)*ltab)+1):(i*ltab)],l1,nsp,
	                        byrow=TRUE,dimnames=list(NULL, 
				     especies=especies))
      }    
      return(result)
}

###################################################################################
# funcion para dibujar (y sacar datos) de la distribucion de riqueza a una escala determinada en una rejilla
# mippp: patron marcado de puntos
      
localdar<- 
function (mippp, mippp.sp=NULL, nx=NULL, ny=NULL, mimark=NULL, idar = "isar", buffer=0, bfw=NULL, 
    r, cross.idar = FALSE, tree = NULL, traits = NULL, namesmark=NULL, correct.trait.na=TRUE, correct.trait = "mean", correct.phylo="mean") 
{
     gridok <- FALSE
     bufferex<-FALSE
     bufferect<- FALSE

    # 1) Check that the column of species is clearly set
    
    if(!is.marked(mippp)) stop ("mapIDAR requires a marked point pattern")
    if (!is.null(namesmark))  mippp$marks <- factor((mippp$marks[namesmark][[1]]))
    
    dmark<-dim(marks(mippp))
      
      if(is.null(namesmark) & !is.null(dmark)){
          if(dmark[2]==1) marks(mippp)<-mippp$marks[,1] else stop(
	  "you should indicate which column of the dataframe of marks
	  stores species names (argument 'namesmark')\n\n") 
      }
      # from here on, mippp has only a vector of marks.




   # 2) Check  th existence of a focal pattern, indicated by "mimark" or "mippp.sp" 
      
   #  if (!is.null(mimark))  if (mimark %in% levels(mippp$marks) == FALSE) {
   #         stop(paste(mimark, " can't be recognized as a mark\n\n\n
   #	    have you indicated in which column of thedataframe are the species\n
   #	    marks? (argument 'namesmark'\n\n"))
   #     }

    # if some mark is given in mimark, we will estimate the idar values around each individual
    
    if(!is.null(mippp.sp))  gridok <- FALSE # it is not a "inner" grid
    if (!is.null(mimark)){
 
           if (mimark %in% levels(mippp$marks) == FALSE) {
               stop(paste(mimark, " can't be recognized as a mark\n\n\n
	       have you indicated in which column of thedataframe are the species\n
	       marks? (argument 'namesmark'\n\n"))
             }
       mippp.sp<- unmark(mippp[mippp$marks==mimark])
       gridok<- FALSE
       
     }

   if(is.null(mippp.sp)){
        if(is.null(nx)& is.null(ny)) nx<-ny<-30 #por defecto 30 x 30 pixels
	if(is.null(nx)& !is.null(ny)) nx<-ny
	if(is.null(nx)& !is.null(ny)) ny<-nx
	
        gridxy <-gridcentres(mippp$window, nx, ny)
	# select only points inside the window
	okbig<-inside.owin(gridxy, w=mippp$window)
        mippp.sp <- ppp(x=gridxy$x[okbig], y=gridxy$y[okbig], window=mippp$window)
	# if we use a regular grid, by default cross.idar is TRUE
      #  cross.idar =TRUE  ##!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	gridok<- TRUE
 #print(mippp.sp)  
   }

    # BUFFER MANAGEMENT#--------------------------------------------------------------------------------------------------

     if(!is.null(bfw)){
           bufferex<- TRUE
           ok<- inside.owin(mippp.sp, w = bfw)
	   mippp.sp0 <- mippp.sp # keep the original focal pattern to return resuilts
	   mippp.sp <- mippp.sp[ok]

     }

   if(buffer!=0 & buffer !="adapt" & is.null(bfw)){ 
         if (is.numeric(buffer) & is.null(bfw)){ 
              if(mippp$window$type!="rectangle") stop("numeric buffer only available for rectangular windows")
              bfw <- owin(mippp$window$xrange + c(buffer, -buffer), 
                      mippp$window$yrange + c(buffer, -buffer))
	bufferect<- TRUE
#print(mippp.sp)	   
	}	      
	if (!gridok){
	   ok<- inside.owin(mippp.sp, w = bfw)
	   mippp.sp0 <- mippp.sp # keep the original focal pattern to return resuilts
	   mippp.sp <- mippp.sp[ok]

	}
	if (gridok){
	   #ok<-inside.owin(gridxy, w=bfw)
	   #mippp.sp <- ppp(x=gridxy$x[ok], y=gridxy$y[ok], window=mippp$window)
	   okbig<-inside.owin(gridxy, w=bfw)
	   mippp.sp <- ppp(x=gridxy$x[okbig], y=gridxy$y[okbig], window=mippp$window)
	}
   }
    
    
        #si el idar es cruzado, se elimina el ppp focal del global
    if (cross.idar == TRUE & gridok ==FALSE) {
        if(is.null(mimark)) stop("for crossed maps you should indicate a focal species (argument 'mimark')")
        mippp<- mippp[mippp$marks!=mimark]
    }

       
	
     if (!is.null(tree)) 
        tree <- checktree(tree = tree, mippp = mippp, idar = idar, 
            correct.phylo = correct.phylo)
    
    if (!is.null(traits)) 
        traits <- checktraits(traits = traits, mippp = mippp, 
            idar = idar, correct.trait.na = correct.trait.na, 
            correct.trait = correct.trait)
	    
    

	
	#*********************************************************************************
	 # calculamos las distancias entre los puntos de la grid y llos arboles 
         # el resultado sera una lista de longitud = length(r) y en cada entrada una tabla de dimension mippp.sp$n X mippp$n
        # con el numero de individuos de cada especie a la distancia r de cada individuo de la especie focal 
		
        cosamt <- mitable(mippp.sp, mippp, r)
    
         #*********************************************************************************
    
    # la O es como en O-ring: anillos para el calculo de rao y cwm
    
    if (idar %in% c("iraodar.O", "icwmar.O")) {
        cosamt.O <- cosamt
        for (i in 2:length(cosamt.O)) cosamt.O[[i]] <- cosamt[[i]] - 
            cosamt[[i - 1]]
        if (buffer == "adapt") {
	    ok<- NULL
            bdp <- bdist.points(mippp.sp)
            for (i in 1:length(r)){
		ok<- cbind(ok, bdp >= r[i])
	        cosamt.O[[i]] <- cosamt.O[[i]][bdp >=  r[i], ]
            }
        }
    }
    if (!idar %in% c("iraodar.O", "icwmar.O")) {
        if (buffer == "adapt") {
	    ok<-NULL
            bdp <- bdist.points(mippp.sp)
            for (i in 1:length(r)){
	         ok<- cbind(ok, bdp >= r[i])
	         cosamt[[i]] <- cosamt[[i]][bdp >= r[i], ]
	     }
        }
    }
    
    
    isar.r <- function(x) {
        Ptj <- function(x) sum(x == 0)/length(x)
        result <- sum(apply(x, 2, function(x) 1 - Ptj(x)))
        return(result)
    }
    cwm.r <- function(x, traits) {
        if (!is.null(dim(traits))) 
            stop("to compute icwmar, 'traits' mut be a vector with the values of juts one trait")
	    
        return(sum(x * traits, na.rm = TRUE)/sum(x))
    }
    
    
    
    
    if (idar == "icwmar") {
    
        micwmar <- sapply(cosamt, function(tr) {
            if (is.null(dim(tr))) 
                tr <- rbind(tr, tr)
      
         #traits<-traits[match( dimnames(tr)[[2]], names(traits))] #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        #cwm.ind <- apply(tr, 1, function(x) cwm.r(x, traits = traits))
	cwm.ind <- apply(tr, 1, function(x) cwm.r(x, traits = traits[match( dimnames(tr)[[2]], names(traits))])) #!!!!!!!!!!!!!!!!!!!!!!!!!OJO EN FUNCIONES INDIVIDUALES: meyter mejor en checktraits
            cwm.ind[is.na(cwm.ind)] <- 0   # OJO: esto tiene que ser 0 o NA ??? !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            #return(mean(cwm.ind))
	    return(cwm.ind)
        })
        result <- micwmar
    }
    if (idar == "icwmar.O") {
        micwmar <- sapply(cosamt.O, function(tr) {
            if (is.null(dim(tr))) 
                tr <- rbind(tr, tr)
            #cwm.ind <- apply(tr, 1, function(x) cwm.r(x, traits = traits))
	    cwm.ind <- apply(tr, 1, function(x) cwm.r(x, traits = traits[match( dimnames(tr)[[2]], names(traits))])) #!!!!!!!!!!!!!!!!!!!!!!!!!OJO EN FUNCIONES INDIVIDUALES: meyter mejor en checktraits
            cwm.ind[is.na(cwm.ind)] <- 0  # OJO: esto tiene que ser 0 o NA ???  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1
            #return(mean(cwm.ind))
	    return(cwm.ind)
        })
        result <- micwmar
    }
    if (idar == "iraodar") {
        miraodar <- sapply(cosamt, function(x) {
            if (is.null(dim(x))) 
                x <- rbind(x, x)
            if (dim(x)[1] < 1) 
                return(NA)
            res <- raoDmap(comm = x, phy = tree)
            return(res)
        })
        result <- miraodar
    }
    if (idar == "iraodar.O") {
        miraodar <- sapply(cosamt.O, function(x) {
            if (is.null(dim(x))) 
                x <- rbind(x, x)
            if (dim(x)[1] < 1) 
                return(NA)
            res <- raoDmap(comm = x, phy = tree)
            return(res)
        })
        result <- miraodar
    }
    if (idar == "ifdar") {
        mifdar <- sapply(cosamt, function(x) {
            if (is.null(dim(x))) 
                x <- rbind(x, x)
            #res <- fdis(x, traits = traits)
	    res <- fdismap(x, traits = traits)
	    
            return(res)
        })
        result <- mifdar
    }
    if (idar == "isar") {
        misar <- sapply(cosamt, function(x) {
            if (is.null(dim(x))) 
                x <- rbind(x, x)
            #res <- isar.r(x)
            res<-apply(x,1,function(y) sum(y>0))
	    
	    return(res)
        })
        result <- misar
    }
    if (idar == "ipsear") {
        mipsear <- lapply(cosamt, function(x) {
            if (is.null(dim(x))) 
                x <- rbind(x, x)
            if (sum(colSums(x) > 0) > 1) 
                res <- pse(x, tree = tree)$PSE
            if (sum(colSums(x) > 0) <= 1) 
                res <- rep(NA, dim(x)[1])
            return(res)
        })
        #result <- sapply(mipsear, mean, na.rm = T)
	result <- mipsear
    }
    if (idar == "ipsvar") {
        mipsvar <- lapply(cosamt, function(x) {
            if (is.null(dim(x))) 
                x <- rbind(x, x)
            if (sum(colSums(x) > 0) > 1) 
                res <- psv(x, tree = tree, compute.var = F)$PSVs
            if (sum(colSums(x) > 0) <= 1) 
                res <- rep(NA, dim(x)[1])
            return(res)
        })
        #result <- sapply(mipsvar, mean, na.rm = T)
	result <-mipsvar
    }
    if (idar == "ipsrar") {
        mipsrar <- lapply(cosamt, function(x) {
            if (is.null(dim(x))) 
                x <- rbind(x, x)
            if (sum(colSums(x) > 0) > 1) 
                res <- psr(x, tree = tree, compute.var = F)$PSR
            if (sum(colSums(x) > 0) <= 1) 
                res <- rep(NA, dim(x)[1])
            return(res)
        })
        #result <- sapply(mipsrar, mean, na.rm = T)
	result <-mipsrar
    }
    if (idar == "ipscar") {
        mipscar <- lapply(cosamt, function(x) {
            if (is.null(dim(x))) 
                x <- rbind(x, x)
            if (sum(colSums(x) > 0) > 1) 
                res <- psc(x, tree = tree)$PSCs
            if (sum(colSums(x) > 0) <= 1) 
                res <- rep(NA, dim(x)[1])
            return(res)
        })
        #result <- sapply(mipscar, mean, na.rm = T)
	result <-mipscar
    }
    if (idar == "imntdar") {
        mimntdar <- lapply(cosamt, function(x) {
            if (is.null(dim(x))) 
                x <- rbind(x, x)
            if (sum(colSums(x) > 0) > 1) 
                res <- mntd(x, dis = tree, abundance.weighted = FALSE)
            if (sum(colSums(x) > 0) <= 1) 
                res <- rep(NA, dim(x)[1])
            return(res)
        })
        #result <- sapply(mimntdar, mean, na.rm = T)
	result <-mimntdar
    }
    
    
    # reconversion a data.frame de nlos resultados tipo matriz, para que puedan ser 
    if(is.matrix(result)) result<- as.data.frame(result)
    
    
  #print("hasta aqui hemos llegado")  
    	# si se ha suministrado o se espera una rejilla #######################################################################33
	if(gridok) {
#cat("okbig=", sum(okbig), "\n")
#cat("ok=", sum(ok), "\n")
	   # if(!is.null(bfw))ok<- inside.owin(gridxy, w=bfw) #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! OJOJOJOJOJOJOJOJOJOJ
	     
	    result.im=list()
	    for (i in 1: length(cosamt)){
	        #mapaidar.ppp<- ppp(x=gridxy$x[ok], y=gridxy$y[ok], window=mippp$window, marks=result)
	        resultgrid=rep(NA,nx*ny)
		# resultgrid[ok] <-result[[i]] 
		#if(buffer!="adapt") resultgrid[ok] <-result[[i]] else resultgrid[ok[,i]] <-result[[i]] 
		#if(buffer!="adapt") resultgrid[okbig][ok] <-result[[i]] else resultgrid[okbig][ok[,i]] <-result[[i]] 
		if(buffer=="adapt") resultgrid[okbig][ok[,i]] <-result[[i]] 
		if(buffer!="adapt" & buffer!=0 & !bufferect) resultgrid[okbig][ok] <-result[[i]] 
		if(bufferect) resultgrid[okbig] <-result[[i]]
		if(bufferex) resultgrid[okbig][ok] <-result[[i]] 
		if(buffer==0 & !bufferex) resultgrid[okbig]<-result[[i]] 
	        result.im[[i]]<- as.im(list(x=unique(gridxy$x),y=unique(gridxy$y),z=matrix(resultgrid,nx,ny)))
	       }
	     #  names(result.im)<- r
	}
	
	# si sehasuministrado o se esperan resultados para un patron focal
	if(!gridok) {
        if(bufferex | bufferect) mippp.sp<- mippp.sp0 # the original non-trimed focal pattern
	result.im=list()
#print(ok)	
#print(result[[1]])
#print(result[[2]])
#print(mippp.sp)
#print(buffer)
#print(bufferex)
        for (i in 1: length(cosamt)){
		marks(mippp.sp)<-NA
		#if(buffer!="adapt")    marks(mippp.sp) [ok]<- result[[i]] else  marks(mippp.sp)[ok[,i]] <-result[[i]] 
		if(buffer=="adapt") marks(mippp.sp)[ok[,i]] <-result[[i]] 
		 if(buffer!="adapt" & buffer!=0 )   marks(mippp.sp) [ok]<- result[[i]]   
		 if(buffer==0 & !bufferex)   marks(mippp.sp) <- result[[i]]
		 if(bufferex) marks(mippp.sp)[ok]<- result[[i]]   
	        #marks(mippp.sp)<- result[[i]]
	        result.im[[i]] <- mippp.sp    
            }
	   #names(result.im)<- r
	 

	}

        names(result.im)<- paste(idar, "_",r* mippp$window$units$multiplier,"_", mippp$window$units$plural, sep="")
	return(result.im)
		
}




#################################################################################
 #funcion necesaria para mapear los componentes individuales de raoD (sin tener en cuenta la ponderacion; podria hacerse opara que devolviera algun valor ponderado por sampl.relabund)
 raoDmap<-
function (comm, phy = NULL) 
{
    res <- list()
    if (is.null(phy)) {
        tij <- 1 - diag(x = rep(1, length(comm[1, ])))
    }
    else {
        #if (class(phy) != "phylo") {  # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! CAMBIADO 04/12/2019 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	if (!inherits (phy, what= "phylo")) {
            #if (!is.matrix(phy) & class(phy) != "dist")  # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! CAMBIADO 04/12/2019 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	     if (!is.matrix(phy) & !inherits (phy, what=  "dist") )
                stop("Phy must be a distance matrix")
            if (is.matrix(phy)) 
                phy <- as.dist(phy)
            dat <- match.comm.dist(comm, phy)
            comm <- dat$comm
            phy <- dat$dist
            tij <- as.matrix(phy/2)
        }
         #if (class(phy) == "phylo") {  # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! CAMBIADO 04/12/2019 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	if (inherits (phy, what= "phylo")) {
            if (!is.ultrametric(phy)) 
                stop("Phylogeny must be ultrametric")
            dat <- match.phylo.comm(phy, comm)
            comm <- dat$comm
            phy <- dat$phy
            tij <- cophenetic(phy)/2
        }
    }
    x <- as.matrix(comm)
    S <- length(x[1, ])
    N <- length(x[, 1])
    total <- apply(x, 1, sum)
    samp.relabund <- total/sum(x)
    x.combined <- matrix(apply(x, 2, sum), nrow = 1)/sum(x)
    x <- sweep(x, 1, total, "/")
    D <- vector(length = N)
    names(D) <- rownames(x)
    for (k in 1:N) D[k] <- sum(tij * outer(as.vector(t(x[k, ])), 
        as.vector(t(x[k, ]))))
    res$Dkk <- D
    res$alpha <- sum(res$Dkk * samp.relabund)
    return(res$Dkk)
}





################################################################################33333



# fdis para los mapas: no devuelve la media sino los valores en cada asbol/punto
fdismap<-
function (comm, traits) 
{
    sp.a <- colSums(comm) > 0
    filas <- dim(comm)[1]
    
    if (sum(sp.a) < 1) 
        return(NA)
    if (sum(sp.a) == 1) 
        return(0)
    if (sum(sp.a) > 1) {
        m.ok <- comm[, sp.a]
        sp.trait.ok <- !is.na(match(rownames(traits), colnames(m.ok)))
        if (!is.null(dim(m.ok))) {
            com.G.0 <- rowSums(m.ok) > 0
            cosad <- as.dist(traits[sp.trait.ok, sp.trait.ok, 
                drop = FALSE])
            
	    if (sum(cosad, na.rm = TRUE) == 0) 
                return(0)
            
	    if (sum(sp.trait.ok) >= 2) {
                com.G.02 <- rowSums(m.ok[, labels(cosad)]) > 
                  0
                cosaf <- fdisp(d = cosad, a = m.ok[com.G.02, 
                  labels(cosad), drop = FALSE], tol = 1e-07)$FDis
		
		result<-rep(NA, filas)
		result[com.G.02]<- cosaf
            }
            
	    if (sum(sp.trait.ok) < 2) {
                cosaf <- fdisp(d = cosad, a = t(as.matrix(m.ok[com.G.0, 
                  labels(cosad)])), tol = 1e-07)$FDis
                result<-rep(NA, filas)
		result[com.G.0]<- cosaf
	    }
            
        }
        if (is.null(dim(m.ok))) 
            result <- NA
        return(result)
    }
}

# es el que se llamaba ipsim4CORR.R


 ipsim<- function(pp, mimark,sigma=0, lambda=NULL, namesmark=NULL){
    # check that in case of multimarked (data.frame) pp, namesmark is provided
    # if mippp has a data,frame of marks, ensure that the name of the column with the labels for species is set in argument "namesmark"
    dimarks <- dim(pp$marks)
    if(!is.null(dimarks) & is.null(namesmark)) stop(paste("For the simulations you should indicate to ipsim argument 'namesmark' which of these ('",paste(names (pp$marks), collapse="' or '"),"') wear the labels for species in the community pp", sep=""))
     if(!is.null(namesmark)){
             nmarks<- names(pp$marks)
            u <- split(pp, f=namesmark)
            # randomize (IPP)) the point pattern of the label
            
            if(!is.null(lambda)){
                  um <-  rpoispp(lambda)
            }
            if(is.null(lambda)){
                if (sigma==0){
                     lambda=u[[mimark]]$n/area.owin(pp$window)# agnadido 29041017 $window
                      um <-  rpoispp(lambda, win=pp$window)
                 }

                if (sigma>0){
                     lambda <- density.ppp(unmark(u[[mimark]]), sigma=sigma)
                     um <-  rpoispp(lambda)
                 }
                     
            } 
            

            um$marks=data.frame(matrix(NA, nrow=um$n,ncol= length(nmarks)))
            names(um$marks) <- nmarks
            um$marks[,namesmark] <- rep(mimark, um$n)
            u[[mimark]] <- um
            split(pp, f=namesmark) <- u
            namesmarks <-names(pp$marks)
      }
     if(is.null(namesmark)){
         # split the multivariate pp
          u <- split(pp)
          # randomize (IPP)) the point pattern of the label

            if(!is.null(lambda)){
                  um <-  rpoispp(lambda)
            }

          if(is.null(lambda)){
                if (sigma==0){
                     lambda=u[[mimark]]$n/area.owin(pp$window) # agnadido 29041017 $window
                      um <-  rpoispp(lambda, win=pp$window)
                 }

                if (sigma>0){
                     lambda <- density.ppp(unmark(u[[mimark]]), sigma=sigma)
                     um <-  rpoispp(lambda)
                 }
                     
            } 

          u[[mimark]] <- um

          # recompose back the splited as a multivariate pp
          split(pp) <- u
      }
      return(pp)
 }

# function to compute percentage acumulators, neutralas and repellers from a list of envelope objects to idar functions
proportion.idar <- function(envlist, alfa = 0.05){

   # envlist: list of envelope objects (one for each focal species)

   allsim.ok <- sapply(envlist, function(x) !is.null(attributes(x)$simfuns))
   # number of usefull species (i.e. with simulated functions)
   
   nsp <-sum(allsim.ok)
   if( nsp <length(envlist)) warning("some of the envelope objects don't wear the simulated functions")

# use  only envelope objects with simulated functions
   envlist <- envlist[allsim.ok]

  nsims<- sapply(envlist, function(x) dim(attributes(x)$simfuns)[2]-1)
  nsim<- nsims[1] #TODO: Check that all envelope objects have the same numbre of simulations !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  r <- envlist[[1]]$r  #TODO: Check that all envelope objects have the same  sequence of r values; use only common r values  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   # get rank of observed function among the simulated functions at each scale r
   scores <- sapply(envlist,
                      function(sp.env) {apply(data.frame(sp.env$obs, data.frame(attributes(sp.env)$simfuns)[,-1]), 1, function(x) rank(x)[1])}
                )
 
      quant <- c(0,1)+c(alfa/2,-alfa/2)
      dataquants <- quantile(0:(nsim+1), probs=quant)

       accumulators <- apply(scores, 2, function(x) x>=dataquants[2])
       repellers    <- apply(scores, 2,  function(x) x<=dataquants[1])
       neutrals         <- apply( scores, 2,  function(x) x>dataquants[1] & x<dataquants[2])

       behaviour <- as.data.frame(neutrals)
       behaviour[accumulators]<- "A"
       behaviour[neutrals]<- "."
       behaviour[repellers]<- "R"
    

       
       percentage <- data.frame(
           p.accumulators = apply(accumulators,1,sum)*100/nsp,
           p.repellers = apply(repellers,1,sum)*100/nsp,
           p.neutrals = apply(neutrals,1,sum)*100/nsp
            )
       
       result <- list(percentage=percentage, nsp=nsp, nsim=nsim, alfa=alfa, r=r,  behaviour= behaviour)
       class(result) = c("pidar", class(result))
       return(result)
       
}

plot.pidar<-function(x,cols=c(1,2,3),type=c("l","o","o"), pch=c(NA,19,19),
                            lty=c(1,1,1),  legend=TRUE, p.legend="topleft",...){

   plot(x$r, x$percentage$p.neutrals, type=type[1], pch=pch[1],col=cols[1], lty=lty[1],
                 ylim=c(0,100), xlab="Radius r of circle (m)", ylab="Percentage of cases",...)
   lines(x$r, x$percentage$p.accumulators, type=type[2], pch=pch[2],col=cols[2],lty=lty[2],...)
   lines(x$r, x$percentage$p.repellers, type=type[3], pch=pch[3], col=cols[3],lty=lty[3],...)
   
   if(legend==TRUE){
        if(length(p.legend)>1){
           xleg<- p.legend[1]
           yleg<- p.legend[2]
           legend(x=xleg,y=yleg, legend=c("neutrals", "accumulators","repellers"),
                                           col=cols, pch=pch, lty=lty)
        }
         if(length(p.legend)==1){
             legend(x=p.legend, legend=c("neutrals", "accumulators","repellers"),
                                           col=cols, pch=pch, lty=lty)
        }
   }
}

`gowdis` <-
function(x, w, asym.bin = NULL, ord = c("podani", "metric", "classic") ){

if (length(dx <- dim(x)) != 2 || !(is.data.frame(x) || is.numeric(x))) stop("x is not a dataframe or a numeric matrix\n")

# n = number of rows, p = number of variables
n <- dx[1]
p <- dx[2]

ord <- match.arg(ord)

varnames <- dimnames(x)[[2]]

# check for weight vector, add equal weights if missing
if (!missing(w)){

	# check if correct class and length
	if (length(w) != p | !is.numeric(w) ) stop("w needs to be a numeric vector of length = number of variables in x\n")
	if (all(w == 0) ) stop("Cannot have only 0's in 'w'\n")
	w <- w / sum(w)
	}
else w <- rep(1, p) / sum(rep(1, p))


if (is.data.frame(x)) {
        type <- sapply(x, data.class)
        }
    else {
        type <- rep("numeric", p)
        names(type) <- colnames(x)
	}

# replace character variables by factors
if (any(type == "character") ) for (i in 1:p) if (type[i] == "character") x[,i] <- as.factor(x[,i])

# check for binary variables
is.bin <- function(k) all(k[!is.na(k)] %in% c(0,1))

bin.var <- rep(NA,p); names(bin.var) <- varnames
for (i in 1:p) bin.var[i] <- is.bin(x[,i])

if (any(type[bin.var] != "numeric")) stop("Binary variables should be of class 'numeric'\n")

type[type %in% c("numeric", "integer")] <- 1
type[type == "ordered"] <- 2
type[type %in% c("factor", "character")] <- 3
type[bin.var] <- 4

# convert asymmetric binary variables, if present
if (!is.null(asym.bin) ){
	if (!all(bin.var[asym.bin])) stop("Asymetric binary variables must only contain 0 or 1\n")
	else type[asym.bin] <- 5
	}
	
type <- as.numeric(type)

# convert factors to their internal numeric codes
x <- data.matrix(x)

# convert ordinal variables to ranks, following Podani (1999)
# or when ord = "classic", convert to numeric
if (any(type == 2) ) {
   if (ord != "classic")   for (i in 1:p) if (type[i] == 2) x[,i] <- rank(x[,i], na.last = "keep")
   else   for (i in 1:p) if (type[i] == 2) x[,i] <- as.numeric(x[,i])
}

# compute the range of each variable (this will only be used for numeric and ordinal variables)
range.Data <- function(v){
	r.Data <- range(v, na.rm = T)
	res <- r.Data[2]-r.Data[1]
	return(res)
	}

range2<- apply(x, 2, range.Data)

# compute Timax, and Timin for each variable (these will only apply to ordinal variables, see Podani [1999], eq. 2b.)
comp.Timax <- function(v){
	Ti.max <- max(v, na.rm = T)
	no.na <- v[!is.na(v)]
	res <- length(no.na[no.na == Ti.max])
	return(res)
	}

Timax <- apply(x, 2, comp.Timax)
	
comp.Timin <- function(v){
	Ti.min <- min(v, na.rm = T)
	no.na <- v[!is.na(v)]
	res <- length(no.na[no.na == Ti.min])
	return(res)
	}

Timin <- apply(x, 2, comp.Timin)

if (ord == "podani") pod <- 1 else pod <- 2

res <- .C("gowdisC", as.double(x), as.double(w), as.integer(type), as.integer(n), as.integer(p), as.double(range2), as.integer(pod), as.double(Timax), as.double(Timin), res = double(n*(n-1)/2), NAOK = T, PACKAGE = "idar")$res

type[type == 1] <- "C"
type[type == 2] <- "O"
type[type == 3] <- "N"
type[type == 4] <- "B"
type[type == 5] <- "A"

if (any(is.na(res) ) ) attr(res, "NA.message") <- "NA's in the dissimilarity matrix!"
attr(res, "Labels") <- dimnames(x)[[1]]
attr(res, "Size") <- n
attr(res, "Metric") <- "Gower"
attr(res, "Types") <- type
class(res) <- "dist"

return(res)
}

 ipsimlist<- function(pp, mimark,listsim){
   # inhomogeneous simulation of a type within a
   # multivariate point pattern
   # listsim: list with simulated pp from simulador2

   listsim <- lapply(listsim, function(x, pp2=pp, mimark2=mimark)   {
                    # First split the multivariate pp
                     u <- split(pp2)
	             # Second, put the fit inhomogenous  simulated IPP
                      u[[mimark2]] <- unmark(x)
	         # recompose back the splited as a multivariate pp
                 split(pp2) <- u
                 return(pp2)
                 }
	        )
   return(listsim)
 }
 
 
 
 
 

\encoding{Latin1}
\name{checktree}
\alias{checktree}
\alias{checktraits}

\title{Check Trait and Phylo Data}
\description{
  Checks consistency of names of species in spatial, phylogenetic and traits data.
}

\usage{
  checktree(tree,  mippp, idar, correct.phylo)
  checktraits(traits, mippp, idar, correct.trait.na, correct.trait)
  
}

\arguments{
 \item{tree}{A phylogenetic tree in \code{phylo} format (\pkg{ape}) or a phylogenetic covariance matrix}
  \item{traits}{A \link[base]{data.frame} of traits, or a distance matrix among species (in \link[stats]{dist} or \link[base]{matrix} format) computed on a data.frame of traits.} 
  \item{mippp}{A multitype (a.k.a. multivariate) marked point pattern . An object with the \link[spatstat.geom]{ppp} format of \pkg{spatstat}, with the names of species as marks.}
  \item{idar}{Character. The name of the idar function to be computed. Either "isar", "ipsvar", "ipsrar", "ipsear", "ipscar", or "imntdar"}
  \item{correct.phylo}{Character. Either \code{"mean"} meaning \emph{"include missing species in the tree with a constant mean phylogenetic covariance"} or \code{"exclude"}, meaning \emph{"exclude missing species in the tree from the analysis"} }
   \item{correct.trait.na}{Logical flag indicating whether \code{NA} values in the matrix of traits should be "corrected": NA values will be assigned the mean trait value.}
   \item{correct.trait}{Character. Either \code{"mean"} or \code{"exclude"}. Species missing in the data.frame of traits will be assigned mean trait values or  will be excluded from the analysis, respectively. }
}

\details{
This functions check for the coincidence of  species in the point pattern and in the phylogenetic or trait data and for the existence of missing data. If \code{correct.phylo="mean"}, species which are absent in the phylogenetic tree or covariance matrix will be included and assigned mean phylogenetic covariance. If \code{correct.phylo="exclude"}, missing species in the tree will be excluded from the analysis (i.e., will not be considered in the computation of the local phylogenetic indices). If \code{correct.trait.na="TRUE"}, NA values for traits in the data.frame of traits will be assigned the mean trait value. If \code{correct.trait="mean"}, missing species in the data.frame of traits will be included and assigned mean trait values.  If \code{correct.trait="exclude"}, missing species in the data.frame of traits will be excluded from the analysis (i.e., will not be considered in the computation of the local functional indices).
}

\section{Warning}{
       The transcription of species names in the multivariate \code{mippp}, in the row names of  the \code{data.frame} of traits (or in the names or \code{dimnames} of  the distance matrices) should be identical. The same applies to the \code{tiplabels} of the phylogenetic tree.
     }


\value{
  \code{checktree} returns a covariance matrix with the appropriate species corrections.   \code{checktraits} returns a data.frame of traits with the appropriate corrections or a Gower distance matrix among the species from the corrected data.frame of traits if   \code{idar="ifdar"}.
}



\author{ Marcelino de la Cruz \email{marcelino.delacruz@urjc.es}}

\keyword{ spatial }

\encoding{Latin1}
\name{fdis}
\alias{fdis}
\title{Average Functional Dispersion}
\description{
  Computes average Functional Dispersion for several communities.
}

\usage{
  fdis(x, traits)
}


\arguments{
  \item{x}{A community data matrix containing the abundances of the species in the different communities. Rows are sites and species are columns.}
   \item{traits}{A distance matrix among species in \code{\link[base]{matrix}} format, i.e. obtained using  \code{\link[base]{as.matrix}}(\emph{d}), where \emph{d} is a  \code{dist} object from \code{\link[FD]{gowdis}}, \code{\link[stats]{dist}} or similar functions.}
}

\details{
This function is a wrap to \code{\link[FD]{fdisp}} in package \pkg{FD}. It manages some of the possible problems that could appear when computing automatically functional dispersion for local communities in \link{ifdar} (mainly "empty" communities, common when computing ifdar(r) for very small \emph{r}'s). It is a kind of internal function that wouldn't be usually called by the user.
}

\value{
  Numeric. The average functional dispersion of the communities in matrix \code{x}
}

\seealso{
      \code{\link[FD]{fdisp}} for a description of the functional dispersion measure (FDis) of \enc{Laliberté and Legendre (2010)}{Laliberte and Legendre (2010)}.
}


\references{

   \enc{LalibertÃ©, E. and  Legendre, P. (2010) A distance-based framework for measuring functional diversity from multiple traits. \emph{Ecology} 91, 299-305.}{Laliberte, E. and  Legendre, P. (2010) A distance-based framework for measuring functional diversity from multiple traits. \emph{Ecology} 91, 299-305.} 

}

\author{ Marcelino de la Cruz \email{marcelino.delacruz@urjc.es}}

\keyword{ spatial }

\encoding{Latin1}
\name{envelope4idar}
\alias{envelope4idar}
\alias{idar2}
\alias{raoDmod}

\title{Compute Simulation Envelopes for IDAR(r) Functions}
\description{
Compute simulation envelopes for IDAR(r) functions.
}

\usage{
  envelope4idar(mippp, mippp.sp.sim, mippp.sp, mimark=NULL,
    namesmark=NULL, r, idar="isar", buffer="adapt", bfw=NULL,
    nsim=NULL, nrank=1, tree = NULL, traits = NULL, 
    cross.idar=FALSE, savefuns=TRUE, correct.phylo="exclude",
    correct.trait.na=FALSE, correct.trait="mean" )
idar2(mippp.sp, mippp, mimark, idar="isar", buffer,bfw, r, 
    cross.idar=FALSE,  tree = NULL, traits = NULL)
    raoDmod(comm, phy = NULL) 
  
}

\arguments{
  \item{mippp}{A multitype (a.k.a. multivariate) marked point pattern. An object with the \link[spatstat.geom]{ppp} format of \pkg{spatstat}.}
  \item{mippp.sp}{Univariate point pattern of the focal species. An object with the \link[spatstat.geom]{ppp} format of \pkg{spatstat}.}
  \item{mippp.sp.sim}{A list with simulations of the focal species point pattern created with \code{\link{simulador2}} }
  \item{mimark}{Character. Name of the focal species in the multitype \code{mippp}.}
  \item{namesmark}{Character. If the marks in \code{mippp} are within a \code{data.frame}, the name of the column with the species names}
  \item{r}{Vector of distances to compute IDAR(r) functions}  
  \item{idar}{Character. The name of the idar function to be computed. Either "isar", "ipsvar", "ipsrar", "ipsear", "ipscar", "icwmar", "icwmar.O",  "iraodar"or "imntdar"}
  \item{buffer}{Character or numeric. Either "adapt" (i.e., compute an adaptive buffer), or a number indicating de width of a fixed buffer area around the plot border}
  \item{bfw}{An \link[spatstat.geom]{owin} object indicating the limits of the buffer area.}
  \item{nsim}{The number of simulations.}
  \item{nrank}{Integer. Rank of the envelope value amongst the \code{nsim} simulated values. A rank of 1 means that the minimum and maximum simulated values will be used.}
  \item{tree}{A phylogenetic tree in \code{phylo} format (\pkg{ape}) or a phylogenetic covariance matrix}
  \item{traits}{A \link[base]{data.frame} of traits, or a distance matrix among species (in \code{\link[stats]{dist}} or \code{\link[base]{matrix}} format) computed on a data.frame of traits.}
  \item{cross.idar}{Logical. If \code{TRUE}, the focal pattern be excluded form the community being measured. }
   \item{savefuns}{Logical flag indicating whether to save all the simulated function values.}
   \item{correct.phylo}{Character. Either \code{"mean"} meaning \emph{"include missing species in the tree with a constant mean phylogenetic covariance"} or \code{"exclude"}, meaning \emph{"exclude missing species in the tree from the analysis"} }
   \item{correct.trait.na}{Logical flag indicating whether \code{NA} values in the matrix of traits should be "corrected": NA values will be assigned the mean trait value.}
   \item{correct.trait}{Character. Either \code{"mean"} or \code{"exclude"}. Species missing in the data.frame of traits will be assigned mean trait values or  will be excluded from the analysis, respectively. }
   \item{comm}{A data.frame with community data}
\item{phy}{a phylogenetic tree in \code{phylo} format (\pkg{ape}) or more probably, a distance matrix.}
}

\details{
In 2007, Wiegand et al. developed the concept of Individual Species-Area Relationship. Basically, this consist in computing species accumulation curves by samping areas with varying radius \eqn{r} around the individual trees of a \emph{focal} species. Here we provide a version of \emph{ISAR(r)} (\code{idar="isar"}), but we also extend this concept to other diversity-area relationships and provide functions to compute individual phylogenetic diversity-area and individual functional diversity-area relationships. The individual phylogenetic functions  are based in Helmus et al. (2007) measures, i.e., phylogenetic species variability (\code{idar="ipsvar"}), phylogenetic species richness (\code{idar="ipsrar"}), phylogenetic species evenness (\code{idar="ipsear"}), and phylogenetic species clustering (\code{idar="ipscar"}). Also, an individual version of the mean nearest taxon distance of Webb et al. (2002) (\code{idar="imntd"}).  The individual functional-diversity function (\code{idar="ifdar"}) is based in the functional dispersion measure (FDis) of \enc{Laliberté and Legendre (2010)}{Laliberte and Legendre (2010)}. Other available functions are based in a weighted community mean of traits (\code{idar="icwmar"}),  on  a verison of Rao quadratic entropy (\code{idar="iraodar"}), and on the same functions but computed in "rings" defined by the neighbour \code{r} values (\code{idar="icwmar.O"} and \code{idar="iraodar.O"}), which would eliminate the "accumulative" or "memory" effect on \emph{ICWMAR(r)} and \emph{IRAODAR(r)}. 

Although recent litterature (e.g., Wiegand and Moloney 2014) suggest that buffer correction is not necessary for this type of statistics, and by default all functions are estimated without buffer (e.g., \code{buffer=0}), several  edge correction coould be employed. For example,  an adaptative buffer correction could be used (\code{buffer="adapt"}), i.e., for each radius \eqn{r}, only individuals of the focal species that are placed at a distance \eqn{>=r} from the border of the plot are considered in the computation of the different meassures. It is also possible to set a fixed buffer width  (e.g., \code{buffer=30}), which will accelerate te computations but will discard many useful trees. It is also possible to provide also a fixed window (in the argument \code{bfw}) to indicate the limits of the buffer area. This could be useful to computing the IDAR(r) functions in different subsets of the original plot (e.g., in different "habitats").

In general, computing envelopes with \code{envelope4idar} is a little faster than using \link[spatstat.explore]{envelope} and the individual functions (e.g., \link{ipsvar}, \link{ifdar}, etc). In addition, \code{envelope4idar} has the possibility of computing "crossed" individual functions, i.e., using a focal species that is not part of the community whose diversities are being measured. This allows evaluating the diversity of e.g., young trees around older trees, etc. This is accomplished by seting the argument \code{cross.idar=TRUE}.

While  \code{envelope4idar} manages data and results, \code{idar2} actually computes the individual functions (both observed and simulated) . In general, \code{idar2} would not be called directly by the user.

\code{raoDmod} is a modification of the function \link[picante]{raoD} of \pkg{picante} to accept distance matrices instead of phylogenetoc trees. It would not be called directly by the user.

}

\section{Warning}{
       The transcription of species names in the multivariate \code{mippp}, in the row names of  the data.frame of traits (or in the names or dimnames of  the distance matrices) should be identical. The same applies to the tiplabels of the phylogenetic tree.
     }


\value{
  An object of class "\code{fv}", see  \link[spatstat.explore]{fv.object}, and \link[spatstat.explore]{envelope},which can be plotted directly using  \link[spatstat.explore]{plot.fv}. 
Essentially a data frame containing columns 
   \describe{
      \item{r}{the vector of values of the argument r at which the \code{idar(r)} function has been estimated}
 
      \item{obs}{values of the summary function for the data point pattern }
 
      \item{lo}{lower envelope of simulations}
 
      \item{hi}{upper envelope of simulations }

       \item{nmean}{estimated theoretical value of the summary function under the considered null model, computed by averaging simulated values}

   }


}

\seealso{
       \code{\link[picante]{psd}} for a description of the phylogenetic measures of Helmus et al. (2007).

        \code{\link[FD]{fdisp}} for a description of the functional dispersion measure (FDis) of \enc{Laliberté and Legendre (2010)}{Laliberte and Legendre (2010)}.
}


\references{

   Helmus M.R., Bland T.J., Williams C.K. and Ives A.R. (2007) Phylogenetic measures of biodiversity. \emph{American Naturalist}, 169, E68-E83.

   \enc{Laliberté, E. and  Legendre, P. (2010) A distance-based framework for measuring functional diversity from multiple traits. \emph{Ecology} 91, 299-305.}{Laliberte, E. and  Legendre, P. (2010) A distance-based framework for measuring functional diversity from multiple traits. \emph{Ecology} 91, 299-305.} 

   Wiegand,T., Gunatilleke, C.V.S., Gunatilleke, I.A.U.N. and  Huth, A. (2007) How individual species structure diversity in tropical forests. \emph{PNAS} 104, 19029-19033.

   Webb, C., Ackerly, D., McPeek, M., and Donoghue M.  2002. Phylogenies and community ecology. \emph{Annual Review of Ecology and Systematics} 33:475-505

}

\author{ Marcelino de la Cruz \email{marcelino.delacruz@urjc.es}}


\examples{



# compute envelope for isar around sp_44
data(SF)
data(SFtraits)
data(SFphylotree)

  
# 1) Create a list with simulations of the focal species point pattern  with simulador2()
#     or by hand. # Beware that each simulated ppp should be marked (with the mark
#      of the focal species)

#  Example for simulations of an inomogenous PP
# Adjust an IPP:
    
    sp_44.ipp<- density.ppp(unmark(SF[SF$marks$species=="sp_44"]))
   
# simulate 19 realizations of the adjusted IPP
# (BEWARE: in real tests you should use 199 or higher)
    sp_44.ipp.sim <- vector(mode="list", le=19)
    sp_44.ipp.sim <- lapply(sp_44.ipp.sim, function(x) x=rpoispp(sp_44.ipp))
   # mark each simulated pattern
    sp_44.ipp.sim.m <- lapply(sp_44.ipp.sim, function(x) 
                                       {marks(x) = factor(rep("sp_44", x$n)); return(x)})

# 2) compute.envelopes
     # ISAR
  isar.sp_44.ipp.env<- envelope4idar(mippp=SF, mippp.sp.sim= sp_44.ipp.sim.m,
                mimark="sp_44", namesmark="species",r=1:30, buffer=0) 
\dontrun{
     # IFDAR
 ifdar.sp_44.ipp.env<- envelope4idar(mippp=SF, mippp.sp.sim= sp_44.ipp.sim.m,
                mimark="sp_44",  namesmark="species", r=1:30, idar="ifdar", buffer=0, 
		 traits=SFtraits, correct.trait.na=TRUE)

    #IPSVAR
   ipsvar.sp_44.ipp.env<- envelope4idar(mippp=SF, mippp.sp.sim= sp_44.ipp.sim.m,
                 mimark="sp_44", namesmark="species", r=1:30, idar="ipsvar", buffer=0, 
		  tree=SFphylotree)

####################################
# Computing CROSS_IDAR
#--------------------------------------------------

# You need a focal pattern that its not part of the multivariate pattern which is "measured"
# to estimate diversity.
# For example, let's measure diversity of small trees  around large trees of the focal species.

# First, obtain the pattern of large and small trees
# the pattern of small trees will be the "measured" one, i.e., the argument "mippp"

SFlarge<- SF[SF$marks$dbh>=10]
SFsmall <- SF[SF$marks$dbh<10]

# pattern of the focal species (this will be the argument "mippp.sp")
sp_44.large<- SFlarge[SFlarge$marks$species=="sp_44"]

# list of simulated patterns of the focal species (e.g., from an IPP)
sp_44.large.ipp<- density.ppp(sp_44.large)
sp_44.large.ipp.sim <- vector(mode="list", le=99)
sp_44.large.ipp.sim <- lapply(sp_44.large.ipp.sim, function(x) x=rpoispp(sp_44.large.ipp))

# COMPUTE envelopes for cross-ISAR(r)
 isar.sp_44.large.cross.ipp.env<- envelope4idar(mippp=SFsmall, r=1:30, buffer=0,
			mippp.sp.sim= sp_44.large.ipp.sim, mippp.sp=sp_44.large,
                        namesmark="species", cross.idar =TRUE) 

# COMPUTE envelopes for cross-IFDAR(r)
ifdar.sp_44.large.cross.ipp.env<- envelope4idar(mippp=SFsmall, r=1:30, idar="ifdar", 
                       buffer=0, mippp.sp.sim= sp_44.large.ipp.sim,  mippp.sp=sp_44.large,
                       namesmark="species", traits=SFtraits, correct.trait.na=TRUE,
                       cross.idar =TRUE) 

# COMPUTE envelopes for cross-IPSVAR(r)
ipsvar.sp_44.large.cross.ipp.env<- envelope4idar(mippp=SFsmall, r=1:30, idar="ipsvar", 
                      buffer=0, mippp.sp.sim= sp_44.large.ipp.sim,  mippp.sp=sp_44.large,
                      namesmark="species",  tree=SFphylotree, cross.idar =TRUE) 


####################################
# Comparing the performance of envelope() and envelope4idar()
# 
#--------------------------------------------------
require(ecespa) # for the ipc.estK() function
data(SF)
SFsp<- unmark(SF)
marks(SFsp)<- SF$marks$species
sp_44.ppp<-unmark(SF[SF$marks$species=="sp_44"])
sp_44.pc<- ipc.estK(sp_44.ppp)


# use multifocalsimulator() to use the same simulations with both functions.
sp_44.pc.sim.mf0<-multifocalsimulator(pp=SFsp, mimark="sp_44",
                           simulate=expression(rIPCP(sp_44.pc)), nsim=99, nmin=sp_44.ppp$n)

# envelopes with function envelope()
gc()
t0<- Sys.time()
  ifdar.sp_44.pc.env.e2<- envelope(SFsp, fun=ifdar, mimark="sp_44", traits=SFtraits,
                            correct.trait.na=TRUE, nsim=99, simulate=sp_44.pc.sim.mf0, r=1:30,
                            savefuns=TRUE, buffer=0)
Sys.time()-t0

# envelopes with function envelope4idar()

# Here you should input simulated patterns only for the focal species so, first,
# extract it fom the list of simulated multivariate ppp

sp_44.pc.sim.mf00<- lapply(sp_44.pc.sim.mf0, function(x) {x=x[x$marks=="sp_44"];return(x)})

gc()
t0<- Sys.time()
ifdar.sp_44.pc.env2<- envelope4idar(mippp=SF, mippp.sp.sim= sp_44.pc.sim.mf00, mimark="sp_44",
				namesmark="species", r=1:30, idar="ifdar", buffer=0, 
                                nsim=99, traits=SFtraits, correct.trait.na=TRUE)
Sys.time()-t0


}
}

\keyword{ spatial }

\encoding{Latin1}
\name{isar}
\alias{isar}
\alias{ipscar}
\alias{ipsear}
\alias{ipsvar}
\alias{ipsrar}
\alias{ifdar}
\title{Individual Diversity Area Relationships}
\description{
  Estimate different Individual Diversity-Area Relationships from a multivariate point pattern.
}

\usage{
  isar(mippp, mippp.sp=NULL, mimark=NULL,  namesmark=NULL, r=NULL,
   buffer=0, bfw=NULL)
  ipscar(mippp, mippp.sp=NULL, mimark=NULL,  namesmark=NULL,
   tree=NULL, r=NULL, buffer=0, bfw=NULL, correct.phylo="mean")
  ipsear(mippp, mippp.sp=NULL, mimark=NULL,  namesmark=NULL,
   tree=NULL, r=NULL, buffer=0, bfw=NULL, correct.phylo="mean")
  ipsvar(mippp, mippp.sp=NULL, mimark=NULL,  namesmark=NULL,
   tree=NULL, r=NULL, buffer=0, bfw=NULL, correct.phylo="mean")
  ipsrar(mippp, mippp.sp=NULL, mimark=NULL,  namesmark=NULL,
   tree=NULL, r=NULL, buffer=0, bfw=NULL, correct.phylo="mean")
  ifdar(mippp, mippp.sp=NULL, mimark=NULL,  namesmark=NULL,
   traits=NULL, r=NULL, buffer=0, bfw=NULL, correct.trait.na=FALSE,
     correct.trait="mean")
}

\arguments{
  \item{mippp}{A multitype (a.k.a. multivariate) marked point pattern. An object with the \link[spatstat.geom]{ppp} format of \pkg{spatstat}.}
  \item{mippp.sp}{Univariate point pattern of the focal species. An object with the \link[spatstat.geom]{ppp} format of \pkg{spatstat}.}
  \item{mimark}{Character. Name of the focal species in the multitype \code{mippp}.}
  \item{namesmark}{Character. If the marks in \code{mippp} are within a \code{data.frame}, the name of the column with the species names}
  \item{buffer}{One of "adapt", i.e., compute an adaptive buffer, or a number indicating de width of a fixed buffer area around the plot border}
  \item{bfw}{An \link[spatstat.geom]{owin} object indicating the limits of the buffer area.}
  \item{r}{Vector of distances to compute IDAR(r) functions}
  \item{tree}{A phylogenetic tree in \code{phylo} format (\pkg{ape}) or a phylogenetic covariance matrix}
  \item{traits}{A \link[base]{data.frame} of traits, or a distance matrix among species (in \code{\link[stats]{dist}} or \code{\link[base]{matrix}} format) computed on a data.frame of traits.}
  \item{correct.phylo}{Character. Either \code{"mean"} meaning \emph{"include missing species in the tree with a constant mean phylogenetic covariance"} or \code{"exclude"}, meaning \emph{"exclude missing species in the tree from the analysis"} }
   \item{correct.trait.na}{Logical flag indicating whether \code{NA} values in the matrix of traits should be "corrected": NA values will be assigned the mean trait value.}
   \item{correct.trait}{Character. Either \code{"mean"} or \code{"exclude"}. Species missing in the data.frame of traits will be assigned mean trait values or  will be excluded from the analysis, respectively. }

}

\details{
In 2007, Wiegand et al. developed the concept of Individual Species-Area Relationship. Basically, this consist in computing species accumulation curves by samping areas with varying radius \eqn{r} around the individual trees of a \emph{focal} species. Here we extend this concept to other diversity-area relationships and provide functions to compute individual phylogenetic diversity-area and individual functional diversity-area relationships. The individual phylogenetic functions are based in Helmus et al. (2007) measures, i.e., phylogenetic species variability (\code{ipsvar}), phylogenetic species richness (\code{ipsrar}), phylogenetic species evenness (\code{ipsear}), and phylogenetic species clustering (\code{ipscar}). The individual functional-diversity function (\code{ifdar}) is based in the functional dispersion measure (FDis) of \enc{Laliberté and Legendre (2010)}{Laliberte and Legendre (2010)}.

Although recent litterature (e.g., Wiegand and Moloney 2014) suggest that buffer correction is not necessary for this type of statistics, and by default all functionsare estimated without buffer (e.g., \code{buffer=0}), several  edge correction coould be employed. For example,  an adaptative buffer correction could be used (\code{buffer="adapt"}), i.e., for each radius \eqn{r}, only individuals of the focal species that are placed at a distance \eqn{>=r} from the border of the plot are considered in the computation of the different meassures. It is also possible to set a fixed buffer width  (e.g., \code{buffer=30}), which will accelerate te computations but will discard many useful trees. It is also possible to provide also a fixed window (in the argument \code{bfw}) to indicate the limits of the buffer area. This could be useful to computing the IDAR(r) functions in different subsets of the original plot (e.g., in different "habitats"). 


}

\section{Warning}{
       The transcription of species names in the multivariate \code{mippp}, in the row names of  the \code{data.frame} of traits (or in the names or \code{dimnames} of  the distance matrices) should be identical. The same applies to the \code{tiplabels} of the phylogenetic tree.
}


\value{
  An object of class "\code{fv}", see  \link[spatstat.explore]{fv.object}, which can be plotted directly using  \link[spatstat.explore]{plot.fv}. 

Essentially a data frame containing a column named \emph{r} with the vector of values of the argument r at which the IDAR(r) function has been estimated and  aonther column, named \emph{"isar", "ipsvar", "ipsrar", "ipsear", "ipscar"} or \emph{"ifdar"}, according to the selected idar argment. This column contains an estimate of the selected IDAR(R) function.

}

\section{Simulation envelopes}{
To compute simulation envelopes for the IDAR(r) functions,  use \code{\link[spatstat.explore]{envelope}}. See the examples in this help page and in \code{\link{ipsim}} to know how to compute simulation envelopes from appropriate null models.

To compute envelopes for "crossed" IDAR(r) functions or to accelerate the computation of "single" IDAR(r) functions, use \code{\link{envelope4idar}}.
}


\seealso{
       \code{\link[picante]{psd}} for a description of the phylogenetic measures of Helmus et al. (2007).

        \code{\link[FD]{fdisp}} for a description of the functional dispersion measure (FDis) of \enc{Laliberté and Legendre (2010)}{Laliberte and Legendre (2010)}.
}


\references{

   Helmus M.R., Bland T.J., Williams C.K. and Ives A.R. (2007) Phylogenetic measures of biodiversity. \emph{American Naturalist}, 169, E68-E83.

   \enc{Laliberté, E. and  Legendre, P. (2010) A distance-based framework for measuring functional diversity from multiple traits. \emph{Ecology} 91, 299-305.}{Laliberte, E. and  Legendre, P. (2010) A distance-based framework for measuring functional diversity from multiple traits. \emph{Ecology} 91, 299-305.} 

   Wiegand,T., Gunatilleke, C.V.S., Gunatilleke, I.A.U.N. and  Huth, A. (2007) How individual species structure diversity in tropical forests. \emph{PNAS} 104, 19029-19033.

}

\author{ Marcelino de la Cruz \email{marcelino.delacruz@urjc.es}}

\examples{

# ISAR
   # Point pattern with a data.frame of marks
   data(SF)
   isar.sp_44 <- isar(mippp = SF, mimark="sp_44", namesmark="species", r=1:40) 
   plot(isar.sp_44)
 
   # Point pattern with just a vector of marks
    data(lansing)
    isar.blackoak <- isar(mippp = lansing, mimark="blackoak",  r=seq(0.01, 0.25, le=100))
    plot(isar.blackoak)

   # Examples of the use of different buffers
     # No  buffer at all (by deffault, buffer = 0)
     isar.sp_44.0 <- isar(mippp = SF, mimark="sp_44", namesmark="species", r=1:18)    

     # Adaptive buffer (for each r, use only points within a r distance form the border)
     isar.sp_44.a <- isar(mippp = SF, mimark="sp_44", namesmark="species", r=1:18,
                                buffer="adapt") 

     # Predefined window, for example with a buffer of 7 m within plot limits
     mibfw<- erosion(SF$win, r=7)
      isar.sp_44.w <- isar(mippp = SF, mimark="sp_44", namesmark="species", r=1:18, bfw=mibfw) 


   ######################
   ### Phylogenetic functions ###
   ######################
   
data(SFphylotree)

# IPSCAR
    ipscar.sp_44 <- ipscar(mippp = SF, mimark="sp_44", namesmark="species", r=1:40,
                                     tree=SFphylotree)
    plot(ipscar.sp_44)
 
# IPSEAR
    ipsear.sp_44 <- ipsear(mippp = SF, mimark="sp_44", namesmark="species", r=1:40,
                                  tree=SFphylotree)
    plot(ipsear.sp_44)

# IPSVAR
    ipsvar.sp_44 <- ipsvar(mippp = SF, mimark="sp_44", namesmark="species", r=1:40,
                                   tree=SFphylotree)
    plot(ipsvar.sp_44)

# IPSRAR
    ipsrar.sp_44 <- ipsrar(mippp = SF, mimark="sp_44", namesmark="species", r=1:40,
                                  tree=SFphylotree)
    plot(ipsrar.sp_44)
 
  #####################
  ###  Functional functions ###
  #####################
 
data(SFtraits)

# IFDAR
   # this will cause an error becuse some species have NA's in the vector of trait values
\dontrun{
  # ifdar.sp_44 <- ifdar(mippp = SF, mimark="sp_44", namesmark="species", traits=SFtraits,
  #                           r=1:40, correct.trait="exclude")

   # "correct" NA's in trait values  by assigning tospecies without traits the average of the trait 
   # for  all the other species
   ifdar.sp_44 <- ifdar(mippp = SF, mimark="sp_44", namesmark="species",  traits=SFtraits,
			      r=1:40, correct.trait.na=TRUE)

    #"correct"  the existence of NA's in trait values  by excluding species without traits from the
    # analysis
    ifdar.sp_44 <- ifdar(mippp = SF, mimark="sp_44", namesmark="species", traits=SFtraits,
                               r=1:40, correct.trait.na=TRUE,correct.trait="exclude")

     plot(ifdar.sp_44)
}
# For examples of envelopes for these functions  see the help page of ipsim() or envelope4idar()

}


\keyword{ spatial }

\name{dummy}
\alias{dummy}
\docType{data}
\title{Dummy Functional Trait Dataset}
\description{
A dummy dataset containing 8 species and 8 functional traits (2 continuous, 2 nominal, 2 ordinal, and 2 binary), with some missing values. Also includes a matrix of species abundances from 10 communities.
}
\usage{
 data(dummy)
 }
\format{
\describe{
\item{trait}{data frame of 8 functional traits on 8 species}
\item{abun}{matrix of abundances of the 8 species from 10 communities}
}
}
\source{
Etienne \enc{Laliberté}{Laliberte} \email{etiennelaliberte@gmail.com}
}
\note{
This dataset has been copied from former package \code{FD} as it was archived on April 15,2026.
}
\keyword{datasets}

\encoding{UTF-8}
\name{LF.gof}
\alias{LF.gof}

\title{Loosmore and Ford Goodness of Fit Test}
\description{ 
  Performs the Loosmore and Ford (2006) test or the Maximum Absolute Deviation test for a spatial point pattern.
}

\usage{
  LF.gof(X, rmin=NULL, rmax=NULL, na.rm=TRUE)
}

\arguments{
  \item{X}{An object resulting from the function \code{\link[spatstat.explore]{envelope}},  i.e., with an attribute "\emph{simfuns}" (obtained using the argument  \code{savefuns=TRUE} in \code{\link[spatstat.explore]{envelope}}) ,which is an object of class "\code{\link[spatstat.explore]{fv}}" containing the summary functions computed for each of the simulated patterns.}
  \item{rmin}{Minimum value of the function argument r over which the maximum absolute deviation, or the integral, will be computed for the test.}
  \item{rmax}{Maximum value of the function argument r over which the maximum absolute deviation, or the integral, will be computed for the test.}
  \item{na.rm}{Should NA's be removed to compute the integral?}
 }

\details{
These function perform a tests for goodness-of-fit of a point pattern dataset to a point process model, based on Monte Carlo simulation from the model. The simulations should have been previously computed with the function \code{\link[spatstat.explore]{envelope}}, applied with the argument \code{savefuns=TRUE} in order to save all the simulated functions, required for the computation of the test.

The test, popularized in the ecological field by Loosmore and Ford (2006) is also described in Diggle (2003, page 14), and according to Baddeley and Turner (2005) also in Diggle (1986) and  Cressie (1991, page 667, equation (8.5.42)).
If the arguments \code{rmin} and \code{rmax} are set to \code{NULL}, the integral of the GoF statistics will be computed over the complete range of r values.
}


\value{
  A list with the following components:
  \describe{
      \item{u}{The GoF statistic, i.e., the value of the integral over the range of \emph{r}'s}
      \item{p}{The p-value of the test}
      \item{na.count.by.r}{Number of \code{NA} values for each r. It helps to evaluate the reliability of the computed u's, specially for small \emph{r}'s}
   }
}

\seealso{
       \code{\link[spatstat.explore]{dclf.test}} for an alternative implementation of the test in \pkg{spatstat}.
}


\references{
Cressie, N.A.C. (1991) \emph{Statistics for spatial data}. John Wiley and Sons, 1991. 

Diggle, P. J. (1986). Displaced amacrine cells in the retina of a rabbit : analysis of a bivariate spatial point pattern. \emph{J. Neuroscience Methods} 18, 115-125. 

Diggle, P.J. (2003) \emph{Statistical analysis of spatial point patterns}, Second edition. Arnold. 

Loosmore, N.B. and Ford, E.D. (2006) Statistical inference using the G or K point pattern spatial statistics. \emph{Ecology} 87, 1925-1931. 

}

\author{ Marcelino de la Cruz \email{marcelino.delacruz@urjc.es}}

\examples{
# some envelopesfor some idar function
# The argument "savefuns" must be set to "TRUE"
# BEWARE: in real tests nsim shpuld be 199 or higher
data(lansing)
maple.lambda<- density(unmark(lansing[lansing$marks=="maple"]))
multi.maple.sim <- multifocalsimulator(lansing, "maple", nsim=10,
                               simulate=expression(rpoispp(maple.lambda)))

isar.maple.env<- envelope(lansing, fun=isar, mimark="maple", nsim=10,
                                     simulate=multi.maple.sim, r=seq(0.01, 0.25, le=100),
                                      savefuns=TRUE)
# Estimate GoF test
LF.gof(isar.maple.env)

}

\keyword{ spatial }

\name{gowdis}
\alias{gowdis}
\title{Gower Dissimilarity}
\description{
\code{gowdis} measures the Gower (1971) dissimilarity for mixed variables, including asymmetric binary variables. Variable weights can be specified. \code{gowdis} implements Podani's (1999) extension to ordinal variables.
}
\usage{
gowdis(x, w, asym.bin = NULL, ord = c("podani", "metric", "classic"))
}
\arguments{
  \item{x}{matrix or data frame containing the variables. Variables can be \code{\link{numeric}}, \code{\link{ordered}}, or \code{\link{factor}}. Symmetric or asymmetric binary variables should be \code{\link{numeric}} and only contain 0 and 1. \code{\link{character}} variables will be converted to \code{\link{factor}}. \code{\link{NA}}s are tolerated.}
  \item{w}{vector listing the weights for the variables in \code{x}. Can be missing, in which case all variables have equal weights.}
  \item{asym.bin}{vector listing the asymmetric binary variables in \code{x}.}
  \item{ord}{character string specifying the method to be used for ordinal variables (i.e. \code{\link{ordered}}). \code{"podani"} refers to Eqs. 2a-b of Podani (1999), while \code{"metric"} refers to his Eq. 3 (see \sQuote{details}); both options convert ordinal variables to ranks. \code{"classic"} simply treats ordinal variables as continuous variables. Can be abbreviated.}
}
\details{
\code{gowdis} computes the Gower (1971) similarity coefficient exactly as described by Podani (1999), then converts it to a dissimilarity coefficient by using \eqn{D = 1 - S}. It integrates variable weights as described by Legendre and Legendre (1998).

Let \eqn{\mathbf{X} = \{x_{ij}\} }{X = {Xij}} be a matrix containing \eqn{n} objects (rows) and \eqn{m} columns (variables). The similarity \eqn{G_{jk}}{Gjk} between objects \eqn{j} and \eqn{k} is computed as

    \deqn{G_{jk} = \frac{\sum_{i=1}^{n} w_{ijk} s_{ijk}}{\sum_{i=1}^{n} w_{ijk}}}{%
    Gjk = sum(Wijk * Sijk) / sum(Wijk)},

where \eqn{w_{ijk}}{Wijk} is the weight of variable \eqn{i} for the \eqn{j}-\eqn{k} pair, and \eqn{s_{ijk}}{Sijk} is the partial similarity of variable \eqn{i} for the \eqn{j}-\eqn{k} pair,

and where \eqn{w_{ijk} = 0}{Wijk = 0} if objects \eqn{j} and \eqn{k} cannot be compared because \eqn{x_{ij}}{Xij} or \eqn{x_{ik}}{Xik} is unknown (i.e. \code{\link{NA}}).


For binary variables, \eqn{s_{ijk} = 0}{Sijk = 0} if \eqn{x_{ij} \neq x_{ik}}{Xij is not equal to Xik}, and  \eqn{s_{ijk} = 1}{Sijk = 1} if \eqn{x_{ij} = x_{ik} = 1}{Xij = Xik = 1} or if \eqn{x_{ij} = x_{ik} = 0}{Xij = Xik = 0}.

For asymmetric binary variables, same as above except that \eqn{w_{ijk} = 0}{Wijk = 0} if \eqn{x_{ij} = x_{ik} = 0}{Xij = Xik = 0}.

For nominal variables, \eqn{s_{ijk} = 0}{Sijk = 0} if \eqn{x_{ij} \neq x_{ik}}{Xij is not equal to Xik} and \eqn{s_{ijk} = 1}{Sijk = 1} if \eqn{x_{ij} = x_{ik}}{Xij = Xik}.

For continuous variables,

    \deqn{s_{ijk} = 1 - \frac{|x_{ij} - x_{ik}|} {x_{i.max} - x_{i.min}} }{%
    Sijk = 1 - [ |Xij - Xik| / (Xi.max - Xi.min) ]}

where \eqn{x_{i.max}}{Xi.max} and \eqn{x_{i.min}}{Xi.min} are the maximum and minimum values of variable \eqn{i}, respectively.


For ordinal variables, when \code{ord = "podani"} or \code{ord = "metric"}, all \eqn{x_{ij}}{Xij} are replaced by their ranks \eqn{r_{ij}}{Rij} determined over all objects (such that ties are also considered), and then


if \code{ord = "podani"}

    \eqn{s_{ijk} = 1}{Sijk = 1} if \eqn{r_{ij} = r_{ik}}{Rij = Rik}, otherwise

    \deqn{ s_{ijk} = 1 - \frac{|r_{ij} - r_{ik}| - (T_{ij} - 1)/2 - (T_{ik} - 1)/2 }{r_{i.max} - r_{i.min} - (T_{i.max} - 1)/2 - (T_{i.min}-1)/2 }}{%
    Sijk = 1 - [ |Rij - Rik| - (Tij - 1) / 2 - (Tik - 1) / 2  / Ri.max - Ri.min - (Ti.max - 1) / 2 - (Ti.min - 1) / 2 ]}

where \eqn{T_{ij}}{Tij} is the number of objects which have the same rank score for variable \eqn{i} as object \eqn{j} (including \eqn{j} itself), \eqn{T_{ik}}{Tik} is the number of objects which have the same rank score for variable \eqn{i} as object \eqn{k} (including \eqn{k} itself), \eqn{r_{i.max}}{Ri.max} and \eqn{r_{i.min}}{Ri.min} are the maximum and minimum ranks for variable \eqn{i}, respectively, \eqn{T_{i,max}}{Ti.max} is the number of objects with the maximum rank, and \eqn{T_{i.min}}{Ti.min} is the number of objects with the minimum rank.


if \code{ord = "metric"}

    \deqn{s_{ijk} = 1 - \frac{|r_{ij} - r_{ik}|}{r_{i.max} - r_{i.min}} }{%
    Sijk = 1 - [ |Rij - Rik| / (Ri.max - Ri.min) ]}

When \code{ord = "classic"}, ordinal variables are simply treated as continuous variables.
}

\value{
an object of class \code{\link{dist}} with the following attributes: \code{Labels}, \code{Types} (the variable types, where 'C' is continuous/numeric, 'O' is ordinal, 'B' is symmetric binary, 'A' is asymmetric binary, and 'N' is nominal), \code{Size}, \code{Metric}.
}
\references{
Gower, J. C. (1971) A general coefficient of similarity and some of its properties. \emph{Biometrics} \bold{27}:857-871.

Legendre, P. and L. Legendre (1998) \emph{Numerical Ecology}. 2nd English edition. Amsterdam: Elsevier.

Podani, J. (1999) Extending Gower's general coefficient of similarity to ordinal characters. \emph{Taxon} \bold{48}:331-340.
}
\author{Etienne \enc{Laliberté}{Laliberte} \email{etiennelaliberte@gmail.com} \ , with some help from Philippe Casgrain for the C interface.}
\note{
\code{fdisp} has been copied from former package \code{FD} as it was archived on April 15,2026.
}
\seealso{\code{\link[cluster]{daisy}} is similar but less flexible, since it does not include variable weights and does not treat ordinal variables as described by Podani (1999). Using \code{ord = "classic"} reproduces the behaviour of \code{\link[cluster]{daisy}}.}
\examples{
data(dummy)
ex1 <- gowdis(dummy$trait)
ex1

# check attributes
attributes(ex1)

# to include weights
w <- c(4,3,5,1,2,8,3,6)
ex2 <- gowdis(dummy$trait, w)
ex2

# variable 7 as asymmetric binary
ex3 <- gowdis(dummy$trait, asym.bin = 7)
ex3

# example with trait data from New Zealand vascular plant species
data(tussock)
ex4 <- gowdis(tussock$trait)
}
\keyword{multivariate}

\name{pisar}
\alias{pisar}
\alias{risar}
\alias{controldis}

\title{
Phylogenetic Individual Species Area Relationship
}
\description{
Compute Phylogenetic Individual Species Area Relationship function, i.e., PISAR(r) and its normalized version rISAR(r).
}
\usage{
pisar(mippp, mippp.sp = NULL, mimark = NULL, namesmark = NULL, d = NULL, r = NULL,
         buffer = 0, bfw = NULL)
risar(mippp, mippp.sp = NULL, mimark = NULL, namesmark = NULL, d = NULL, d0 = NULL,
        r = NULL, buffer = 0, bfw = NULL)
controldis(d, m, mimark)
}


\arguments{
  \item{mippp}{A multitype (a.k.a. multivariate) marked point pattern. An object with the \link[spatstat.geom]{ppp} format of \pkg{spatstat}.}
  \item{mippp.sp}{Univariate point pattern of the focal species. An object with the \link[spatstat.geom]{ppp} format of \pkg{spatstat}.}
  \item{mimark}{Character. Name of the focal species in the multitype \code{mippp}.}
  \item{namesmark}{Character. If the marks in \code{mippp} are within a \code{data.frame}, the name of the column with the species names.}
  \item{buffer}{One of "adapt", i.e., compute an adaptive buffer, or a number indicating the width of a fixed buffer area around the plot border.}
  \item{bfw}{An \link[spatstat.geom]{owin} object indicating the limits of the buffer area.}
  \item{r}{Vector of distances to compute PIDAR(r) functions.}
  \item{d}{A matrix expresing relationships (usually functional or phylogenetic) between species present in the multivariate point pattern.}
  \item{d0}{Another matrix expresing relationships (usually functional or phylogenetic) between species present in the multivariate point pattern.}
  \item{m}{A community (sites x species) data table.}
}
\details{
 The original definition of \emph{ISAR}(\emph{r}) (Wiegand et al. 2007) was reformulated as:

\deqn{ISAR_f (r) =\sum_{m=1}^{S} \delta_{fm} D_{fm} (r)}{PISAR[f](r) = sum[m] delta[f,m]D[f,m](r)}
(Wiegand and Moloney 2014; Wang et al. 2016), where \eqn{D_{fm}(r)}{D[f,m](r)} describe the probabilities that the nearest species \emph{m} neighbor of the typical
individual of the focal species \emph{f} is located within distance \emph{r}, and \eqn{\delta_{fm} }{delta[f,m]} yields a value of zero if \emph{f = m} and a value of one otherwise (note that in their original proposal ISAR was formulated as if the value assigned to \eqn{\delta_{fm} }{delta[f,m]} were 1 for all species pairs, including \emph{f = m}). Based in  this re-formulation, they defined the \emph{Phylogenetic Individual Species Area Relationship}, i.e., \emph{PISAR}(\emph{r}), as:


\deqn{PISAR_f (r) =\sum_{m=1}^{S} \delta_{fm}^{phy}D_{fm} (r)}{PISAR[f](r) = sum[m] delta[f,m]^phy D[f,m] }

where \eqn{\delta_{fm}^{phy}}{delta[f,m]^phy} is an index of phylogenetic (or functional) dissimilarity between species \emph{f}
and \emph{m}. \emph{PISAR}(\emph{r}) quantifies the expected phylogenetic (or functional) diversity of species within the
neighborhood with radius \emph{r} around the typical individual of the focal species \emph{f}.

 They also defined \emph{rISAR}(\emph{r}) as a  a function that is independent of local species richness within the neighborhood \emph{r}; for this, they divided the PISAR function by the ISAR function:

\deqn{rISAR_f (r) = \frac{\sum_{m=1}^{S} \delta_{fm}^{phy}D_{fm} (r)}{\sum_{m=1}^{S} \delta_{fm}D_{fm} (r)}  }{rISAR[f](r) = (sum[m] delta[f,m]^phy D[f,m]) /(sum[m] delta[f,m] D[f,m])}


If the placement of the focal species \emph{f} is unrelated with functional or phylogenetic relationships with their neighbors, the \eqn{rISAR_f(r)}{rISAR[f](r)} will approximate the mean pairwise functional (or phylogenetic) dissimilarity 
\eqn{\Delta_f^P = \sum_m \delta_{fm}^{phy}/(S-1)}{Delta{f}^P = sum{m} delta[f,m]^phy /(S-1)} between an individual of the
focal species \emph{f} and all other species in the plot.


The function \code{controldis} controls that the order of species in the phylogenetic distance matrix matches the order of species amomng the levels of species marks in the point pattern, and extracts the vector of distances from all species to the focal one (\code{mimark}).  


}
\value{
  \code{pisar} and \code{risar} return an object of class \code{fv}, see  \link[spatstat.explore]{fv.object}, which can be plotted directly using  \link[spatstat.explore]{plot.fv}. 

Essentially a data frame containing a column named \emph{r} with the vector of values of the argument r at which the PISAR(r) or rISAR(r) function had been estimated and  another column, named \emph{"risar"} or \emph{"pidar"},which  contains an estimate of the selected function.

\code{controldis} returns either the vector of distances between the focal and the rest of species or a vector of 1's if there is not phyloegenetic distance provided.

}

\section{Simulation envelopes}{
To compute simulation envelopes for  \code{pisar} or \code{risar} functions,  use \code{\link[spatstat.explore]{envelope}}. See the examples in this help page and in \code{\link[idar]{multifocalsimulator}} to know how to compute simulation envelopes from appropriate null models.
}


\references{
Wang, X., et al. (2016). Stochastic dilution effects weaken deterministic effects of niche-based processes in species rich forests. \emph{Ecology, in press}

Wiegand,T., Gunatilleke, C.V.S., Gunatilleke, I.A.U.N. and  Huth, A. (2007) How individual species structure diversity in tropical forests. \emph{PNAS} 104, 19029-19033.

Wiegand, T., and K.A. Moloney. (2014). \emph{A handbook of spatial point pattern analysis in ecology}. Chapman and Hall/CRC press, Boca Raton, FL 
}
\author{ Marcelino de la Cruz \email{marcelino.delacruz@urjc.es}}


 \section{NOTE}{
 When computing risar it is necessary to provide a phylogenetic or functional distance matrix to the argument \code{d}. By default,  argument \code{d0} will be set to a vector of 1's. It is however possible to provide a different matrix to \code{d0} and compute instead, e.g.,  a ratio of phylogenetic to functional diversity.
 
 }

\seealso{
See also \code{\link{isar}} for other individual diversity area functions.
}
\examples{

data(SF)
data(SFphylotree)

# Discard the size mark and keep the species mark in SF ppp:
sfsp<- ppp(SF$x, SF$y, window=SF$window, marks=SF$marks$species)

# compute phylogenetic distance among species
dphy <- cophenetic(SFphylotree)

# compute and plot PISAR function for sp_44
pisar_44 <- pisar(sfsp, mimark="sp_44", r=1:15,  d=dphy)
plot(pisar_44)

\dontrun{

# Compute rISAR and plot envelopes for an inhomogeneous Poisson model
#  of each species in San Francisco plot 
# BEWARE: THIS TAKES QUITE A FEW MINUTES !!!

# Split sfsp point pattern ppp by species
sfsp.sp<- split(sfsp)

#  Species with >= 10 individuals
sfsp10 <- sapply(sfsp.sp, function(x) x$n>=10)
#names of those species
nombressf<- names(sfsp10[sfsp10])

# parameters for the simulations, estimation of intensity, etc.
nsim<-199
nmin<-10
sigma <- 8
r<- seq(1,15, by=0.5)

# list to store results
risar.sf<- list()
# start computation
for( sp in nombressf){
   print(sp)
   # estimate intensity of the focal species
   lambda<- density(unmark(sfsp[sfsp$marks==sp]), sigma=sigma, positive=TRUE)      
   # obtain simulated patterns were all species ecept the focal remain fixed 
   #  and the focal varies according to an inhomomgeneous Poiisson process 
   simulados<- multifocalsimulator(sfsp, mimark=sp,
              simulate=expression(rpoispp(lambda)), nsim=nsim,nmin=nmin)
   # compute risar
   risar.sf[[sp]] <- envelope(sfsp, risar, mimark=sp, d=dphy, r=r,
        simulate=simulados,nsim=nsim, savefuns=T, buffer=0)
}

# plot the results
 dev.new(height=7, width=16)
 par(mfrow=c(3,9))
 for(i in 1:27) plot(risar.sf[[i]], legend=F, main=nombressf[i])

}


}

\keyword{ spatial }

\name{midar}
\alias{midar}
\title{
Customize the Individual Diversity-Area Relationship Function
}
\description{
A wrapper to develop new Individual Diversity-Area Relationship functions on the fly.
}
\usage{
midar(mippp, mippp.sp = NULL, mimark = NULL, namesmark = NULL, traits = NULL,
tree = NULL, r = NULL, buffer = 0, bfw = NULL, what = NULL)
}

\arguments{
 \item{mippp}{A multitype (a.k.a. multivariate) marked point pattern. An object with the \link[spatstat.geom]{ppp} format of \pkg{spatstat}.}
  \item{mippp.sp}{Univariate point pattern of the focal species. An object with the \link[spatstat.geom]{ppp} format of \pkg{spatstat}.}
  \item{mimark}{Character. Name of the focal species in the multitype \code{mippp}.}
  \item{namesmark}{Character. If the marks in \code{mippp} are within a \code{data.frame}, the name of the column with the species names}
  \item{buffer}{One of "adapt", i.e., compute an adaptive buffer, or a number indicating the width of a fixed buffer area around the plot border}
  \item{bfw}{An \link[spatstat.geom]{owin} object indicating the limits of the buffer area.}
  \item{r}{Vector of distances to compute IDAR(r) functions}
  \item{tree}{A phylogenetic tree in \code{phylo} format (\pkg{ape}) or a phylogenetic covariance matrix}
  \item{traits}{A \link[base]{data.frame} of traits, or a distance matrix among species (in \code{\link[stats]{dist}} or \code{\link[base]{matrix}} format) computed on a data.frame of traits.}
  \item{what}{A valid R expression or function that would accept a community matrix (sites x species) and return a unique value}
}
\details{
\code{midar} allows computing new \emph{IDAR}(\emph{r}) functions. The basis of all idar functions are the local communities defined around each point (e.g., each tree) of a focal species (\code{mimark}) for a certain circular neighborhood of radius \emph{r}. Some diversity measure is computed on each community an the average of all of them is returned as the idar value for this \emph{r}, i.e., as \emph{IDAR}(\emph{r}). The function \code{midar} applies the R function or expression defined by the argument \code{what} to each of the  "community data tables" (matrices) generated by \code{\link{mitable}} (one for each \emph{r} interval defined by the argument \code{r}) and return the result as a spatial summary function like all the others in the \pkg{idar} package (e.g., \code{\link{isar}} or \code{\link{pisar}}). The R expression or function should accept a community matrix (sites x species) as input and return a unique numeric value.
}
\value{
  \code{midar} return an object of class \code{fv}, see  \link[spatstat.explore]{fv.object}, which can be plotted directly using  \link[spatstat.explore]{plot.fv}. 
Essentially, a data frame containing a column named \emph{r} with the vector of values of the argument r at which the proposed function had been estimated and  another column, named \emph{"midar"} which  contains an estimate of the selected function.
}
\section{Simulation envelopes}{
To compute simulation envelopes for  \code{midar}  functions,  use \code{\link[spatstat.explore]{envelope}}. See the examples in this help page and in \code{\link[idar]{multifocalsimulator}} to know how to compute simulation envelopes from appropriate null models.
}
\author{ Marcelino de la Cruz \email{marcelino.delacruz@urjc.es}}

\seealso{
\code{\link{isar}} or \code{\link{pisar}} in this package.
}
\examples{

data(SF)
data(SFphylotree)
 # Discard the size mark and keep the species mark in SF ppp:
  sfsp<- ppp(SF$x, SF$y, window=SF$window, marks=SF$marks$species)

# compute "individual Simpsom diversity-area relationship" around sp_44
# using function diversity from package vegan
require(vegan)
simpsom_sp_44<- midar(sfsp, mimark="sp_44", what =function(x) mean(diversity(x, "simpson")),
                                  r=1:15)


# test "individual Simpsom diversity-area relationship" against an inhomogeneous Poisson
# null modellfor sp_44. 
  # estimate intensityh surface for sp_44
   lambda<- density.ppp(unmark(sfsp[sfsp$marks=="sp_44"]), positive=TRUE)
  
  # generate 19 realizations of the null model, keeping all the other species fixed in their
  #  original coordinates.
   simulados<- multifocalsimulator(sfsp, mimark="sp_44", simulate=expression(rpoispp(lambda)),
                      nsim=19, nmin=15)

  # for simplicity define the function that we want to apply to each "local community"
  #  around each tree of sp_44
  mean_simpsom <- function(x) mean(diversity(x, "simpson"))

  # compute envelopes and plot them.
  simpsom_sp_44.env<-envelope(sfsp, midar, mimark="sp_44", what =mean_simpsom ,r=1:15,
                      nsim=19, simulate=simulados)
  plot(simpsom_sp_44.env)


# compute IPSVAR(r) "by hand"
   # first, check tree as would check it ipsvar
   arbol <- checktree(SFphylotree,  SF, "ipsvar", correct.phylo="exclude")

    # define function to obtain the averge psv from the set of local communities
    #  at each neigborhood radii r
     mipsv <- expression(mean(psv(x, tree=tree, compute.var=FALSE)$PSVs, na.rm=TRUE))
     # compute ipsvar "by hand" 
     sp_44_psv<- midar(sfsp, mimark="sp_44", tree=arbol, what =mipsv ,r=1:15)
     plot(sp_44_psv)

# compare it with the result of the built-in function
     plot(ipsvar(sfsp, mimark="sp_44", tree=arbol, r=1:15), add=TRUE, col="blue")
    
}

\keyword{ spatial }

\encoding{Latin1}
\name{mitable}
\alias{mitable}

\title{Tabulate Marks in Neighbourhood of Every Point in a Point Pattern}
\description{
  For a sequence of radii defining different neighbourhood sizes, visit each point in a focal point pattern, find the neighbouring points in a target point pattern, and compile a frequency table of the marks of these neighbour points for each radii.
}

\usage{
  mitable(ppp1,ppp2,r)
  }

\arguments{
  \item{ppp1}{"Focal" point pattern (an \code{\link[spatstat.geom]{ppp}} object of \pkg{spatstat}) or a \link[base]{data.frame} with columns named "x" and "y"}
  \item{ppp2}{"Target" multivariate point pattern (an \code{\link[spatstat.geom]{ppp}} object of \pkg{spatstat}) or a \link[base]{data.frame} with columns named "x", "y", and "marks". The column "marks" wears the (species) name of each point}
   \item{r}{Vector with the sequence of radii (\emph{r>0}) that define different neiborhood sizes.}
}

\details{
Given both a "focal" and a "target" point patterns, \code{mitable} visits each point in the focal point pattern, finds the neighbouring points in the target point pattern, and compile a frequency table of the marks of these neighbour points, where the neibourhood is defined by circles orf radius \emph{r} around the focal points. From an ecological point of view, it provides a \enc{\emph{relevé}}{\emph{releve}} of the local community around each focal tree. It repeats the process for each provided \emph{r}.
}


\value{
  A list, with length = \emph{length(r)}. Within each element of the list, a matrix with dimensions \emph{np} X \emph{nsp}, where \emph{np} is the number of points of the focal point pattern and \emph{nsp} is the number of unique species (i.e., unique marks) in the target point pattern. Cell values in the matrix represent number of neighbours of the \emph{j} species for individual \emph{i} at the considered neibourhood size.
}
\seealso{
  \code{\link[spatstat.explore]{marktable}} for a similar function.
}

\author{ Marcelino de la Cruz \email{marcelino.delacruz@urjc.es}}

\examples{

# Compute number of trees of different species  within circles of several  radii r 
# around each individual maple tree in lansing woods

data(lansing)

maple<- unmark(lansing[lansing$marks=="maple"])
maple
r= c(0.05, 0.08, 0.1)

counts <- mitable(ppp1=maple, ppp2=lansing, r=r) 
dim(counts[[1]])
head(counts[[1]])


}

\keyword{ spatial }

\name{tussock}
\alias{tussock}
\concept{New Zealand}
\concept{functional composition}
\concept{functional trait}
\concept{functional diversity}
\docType{data}
\title{Functional Composition of Short-Tussock Grasslands}
\description{
\code{tussock} contains data on 16 functional traits measured on 53 vascular plant species from New Zealand short-tussock grasslands. It also contains the relative abundances (percent cover) of these 53 species from 30 8x50-m plots.
}
\usage{
  data(tussock)
}
\format{
\code{tussock} is a list of 2 components:
\describe{
\item{trait}{data frame of 16 functional traits measured on 53 plant species: growth form (sensu Cornelissen et al. 2003), reproductive plant height (\eqn{\mathrm{m}}{m}), leaf dry matter content (\eqn{\mathrm{mg} \; \mathrm{g}^{-1}}{mg g^-1}), leaf nitrogen concentration (\eqn{\mathrm{mg} \; \mathrm{g}^{-1}}{mg g^-1}), leaf phosphorous concentration (\eqn{\mathrm{mg} \; \mathrm{g}^{-1}}{mg g^-1}), leaf sulphur concentration (\eqn{\mathrm{mg} \; \mathrm{g}^{-1}}{mg g^-1}), specific leaf area (\eqn{\mathrm{m}^2 \; \mathrm{kg}^{-1}}{m^2 kg^-1}), nutrient uptake strategy (sensu Cornelissen et al. 2003), Raunkiaer life form, clonality, leaf size (\eqn{\mathrm{mm}^2}{mm^2}), primary dispersal mode, seed mass (\eqn{\mathrm{mg}}{mg}), resprouting capacity, pollination syndrome, and lifespan (an ordinal variable stored as \code{\link{ordered}}).}

\item{abun}{matrix containing the relative abundances (percent cover) of the 53 species in 30 plots}
}
}
\details{
The functional traits were measured using standardized methodologies (Cornelissen et al. 2003). Each of the 30 experimental plots from which species cover was estimated is 8x50 m. Relative abundances of all vascular plant species were estimated in November 2007. To do so, 20 1x1-m quadrats per plot were randomly positioned along two longitudinal transects and cover of each species was estimated using a modified Braun-Blanquet scale. This data was pooled at the plot scale to yield the percent cover data.
}
\source{
Etienne \enc{Laliberté}{Laliberte} \email{etiennelaliberte@gmail.com}

\ 
}
\note{
This dataset has been copied from former package \code{FD} as it was archived on April 15,2026.
}
\references{
Cornelissen, J. H. C., S. Lavorel, E. Garnier, S. Diaz, N. Buchmann, D. E. Gurvich, P. B. Reich, H. ter Steege, H. D. Morgan, M. G. A. van der Heijden, J. G. Pausas and H. Poorter. (2003) A handbook of protocols for standardised and easy measurement of plant functional traits worldwide. \emph{Australian Journal of Botany} \bold{51}:335-380.


\enc{Laliberté}{Laliberte}, E., Norton, D. A. and D. Scott. (2008) Impacts of rangeland development on plant functional diversity, ecosystem processes and services, and resilience. \emph{Global Land Project (GLP) Newsletter} \bold{4}:4-6.

Scott, D. (1999) Sustainability of New Zealand high-country pastures under contrasting development inputs. 1. Site, and shoot nutrients. \emph{New Zealand Journal of Agricultural Research} \bold{42}:365-383.
}
\keyword{datasets}

\encoding{UTF-8}
\name{proportion.idar}
\alias{proportion.idar}
\alias{plot.pidar}


\title{Determine the Proportion of Accumulator and Repeller Species}
\description{
  This function determines the proportion of species in a community that "accumulate" or "repell" diversity, in the sense of Wiegand et al. (2007).
}

\usage{
  proportion.idar(envlist, alfa = 0.05)
     \method{plot}{pidar}(x,cols=c(1,2,3),type=c("l","o","o"), pch=c(NA,19,19),
    lty=c(1,1,1),  legend=TRUE, p.legend="topleft",...)
}

\arguments{
  \item{envlist}{A list whose elements are the result of applying \link[spatstat.explore]{envelope} to a set of point patterns.}
  \item{alfa}{alpha value to calculate deviations from null model}
  \item{x}{The result of \code{proportion.idar}}
  \item{cols}{A vector (length =3) with the color names or codes for the curves of each of the 3 categories: "neutral", "accumulator", "repeller".}
  \item{type}{A vector (length =3), with the type of plot for each category. See \link[graphics]{plot.default}.}
  \item{pch}{A vector (length =3), with the point type  for each category. See \link[graphics]{points}}
  \item{lty}{A vector (length =3), with the line type  for each category. See \link[graphics]{par}}
   \item{legend}{Logical. Should a legend be added to the plot?}
  \item{p.legend}{Position of the legend. It can be a vector wit the xycoordinates of the upperleft corner of the legend box or a  keyword accepted by \code{\link[graphics]{legend}}}
  \item{...}{Additional graphical parameters passed both to funtions \code{plot} and \code{lines}}
}

\details{
This function determines the proportion of accumulator, repeller and neutral species at each scale \emph{r}, following the approach of Wiegand et al (2007). A species is classified as an accumulator at scale \emph{r} if there are less than \eqn{(nsim+1) *alpha/2} simulated values greater than the observed \emph{idar(r)}. On the contrary, a species is classified as repeller at scale if there are less than \eqn{(nsim+1) *alpha/2} simulated values smaller than the observed \emph{idar(r)}.  The percentage is computed over the total of species in\code{ envlist}. It is necessary that the objects in envlist (i.e. the envelope objects) had been computed with the argument "\code{simfuns=TRUE}".

}
\value{
\code{proportion.idar} produces an object of class "\code{pidar}", basically a list with components

   \describe{
        \item{percentage}{A \code{data.frame} with 3 columns ("p.accumulators", "p.repellers", "p.neutrals") indicating the percentage in each category for each \emph{r} (rows)}
        \item{nsp}{The total number of species for which the percentage has been computed.}
        \item{nsim}{The number of simulations}
        \item{alfa}{The alpha value employed to calculate the deviations from null model and the assignation to each of the categories}
        \item{r}{The vector of r values at which the idar(r) functions have been estimated}
         \item{behaviour}{A data.frame with the behaviour ("A"= accumulator, "R" = repeller) of each species at each radius r}
    }
   
   \code{plot.pidar} plots the result.

}
\references{
   Wiegand,T., Gunatilleke, C.V.S., Gunatilleke, I.A.U.N. and  Huth, A. (2007) How individual species structure diversity in tropical forests. \emph{PNAS} 104, 19029-19033.

}

\author{ Marcelino de la Cruz \email{marcelino.delacruz@urjc.es}}

\examples{

# Compute percentage of accumulator, neutral an repeller species (for ISAR) in Lansing woods
# In this example, against a null model of IPP.
data(lansing)

# Compute rISAR and plot envelopes for an inhomogeneous Poisson model
# of each species in San Francisco plot
# Split sfsp point pattern ppp by species
lansing.sp<- split(lansing)

# Set parameters for the simulations, estimation of intensity surface for IPP, etc.
# BEWARE: THIS is for R-TESTING ALONE. REAL TESTS SHOULD EMPLOY 199 simulations or higher
nsim<-5
r<- seq(0.01,0.25, by=0.01)
# Create list to store the results
isar.lansing<- list()
# start computation

for( i in 1: length(lansing.sp)){
   print(i)
   # estimate intensity of the focal species
   lambda<- density(lansing.sp[[i]])
   # obtain simulated patterns were all species except the focal remain fixed
   # and the focal varies according to an inhomomgeneous Poiisson process
   simulados<- multifocalsimulator(lansing, mimark=levels(lansing$marks)[i],
                            simulate=expression(rpoispp(lambda)), nsim=nsim)
  # compute isar and envelopes for each species
  # It is COMPULSORY that the argument "savefuns" it is set to TRUE
   isar.lansing[[i]] <- envelope(lansing, isar, mimark=levels(lansing$marks)[i],  r=r,
         simulate=simulados,nsim=nsim, savefuns=TRUE, buffer=0)
}


prop.isar.lansing <- proportion.idar(isar.lansing)
head(prop.isar.lansing $percentage)
head(prop.isar.lansing $behaviour)
plot(prop.isar.lansing, p.legend=c(0.15,60))

\dontrun{

# Compute percentage of accumulator, neutral an repeller species (for ISAR) in San Francisco
# forest.  In this example, against a null model of IPP.
data(SF)

# Discard the size mark and keep the species mark in SF ppp:
sfsp<- ppp(SF$x, SF$y, window=SF$window, marks=SF$marks$species)

# Compute ISAR and plot envelopes for an inhomogeneous Poisson model
# of each species in San Francisco plot
# Split sfsp point pattern ppp by species
sfsp.sp<- split(sfsp)
# Select species with >= 10 individuals (to get some statistical power)
sfsp10 <- sapply(sfsp.sp, function(x) x$n>=10)
#Get names of those species
nombressf<- names(sfsp10[sfsp10])
# Set parameters for the simulations, estimation of intensity surface for IPP, etc.
nsim<-199
nmin<-10
sigma <- 8
r<- seq(1,15, by=0.5)
# Create list to store the results
isar.sf<- list()
# start computation
# BEWARE: THIS TAKES QUITE A FEW MINUTES!!!
for( sp in nombressf){
   print(sp)
   # estimate intensity of the focal species
   lambda<- density(unmark(sfsp[sfsp$marks==sp]), sigma=sigma, positive=TRUE)
   # obtain simulated patterns were all species except the focal remain fixed
   # and the focal varies according to an inhomomgeneous Poiisson process
   simulados<- multifocalsimulator(sfsp, mimark=sp,
                            simulate=expression(rpoispp(lambda)), nsim=nsim,nmin=nmin)
  # compute isar and envelopes for each species
  # It is COMPULSORY that the argument "savefuns" it is set to TRUE
   isar.sf[[sp]] <- envelope(sfsp, isar, mimark=sp,  r=r,
         simulate=simulados,nsim=nsim, savefuns=TRUE, buffer=0)
}


prop.isar.SF <- proportion.idar(isar.sf)
head(prop.isar.SF$percentage)
head(prop.isar.SF$behaviour)
plot(prop.isar.SF)

}
}

\keyword{ spatial }

\name{localdar}
\alias{localdar}
\alias{fdismap}
\alias{raoDmap}
\title{
Map Local Diversity Area Relationships
}
\description{
Estimates and maps local diversity-area relationships.
}
\usage{
localdar(mippp, mippp.sp = NULL, nx = NULL, ny = NULL, mimark = NULL, idar = "isar", 
            buffer = 0, bfw = NULL, r, cross.idar = FALSE, tree = NULL, traits = NULL, 
            namesmark = NULL, correct.trait.na = TRUE, correct.trait = "mean",
            correct.phylo="mean")
fdismap(comm, traits)
raoDmap(comm, phy = NULL)
}

\arguments{
 \item{mippp}{A multitype (a.k.a. multivariate) marked point pattern. An object with the \link[spatstat.geom]{ppp} format of \pkg{spatstat}.}
 \item{mippp.sp}{Univariate point pattern of the focal species. An object with the \link[spatstat.geom]{ppp} format of \pkg{spatstat}.}
 \item{nx}{Number of points of the grid along the x axis.}
 \item{ny}{Number of points of the grid along the y axis.}
 \item{mimark}{Character. Name of the focal species in the multitype \code{mippp}.}
 \item{idar}{Character. The name of the idar function to be computed. Either "isar", "ipsvar", "ipsrar", "ipsear", "ipscar", "icwmar", "icwmar.O",  "iraodar"or "imntdar"}
 \item{buffer}{One of "adapt", i.e., compute an adaptive buffer, or a number indicating the width of a fixed buffer area around the plot border}
 \item{bfw}{An \link[spatstat.geom]{owin} object indicating the limits of the buffer area.}
 \item{r}{Vector of distances to compute IDAR(r) functions}
 \item{cross.idar}{Logical. If \code{TRUE}, the focal pattern will be excluded form the community being measured. }
  \item{tree}{A phylogenetic tree in \code{phylo} format (\pkg{ape}) or a phylogenetic covariance matrix}
  \item{traits}{A \link[base]{data.frame} of traits, or a distance matrix among species (in \code{\link[stats]{dist}} or \code{\link[base]{matrix}} format) computed on a data.frame of traits.}
  \item{namesmark}{Character. If the marks in \code{mippp} are within a \code{data.frame}, the name of the column with the species names}
  \item{correct.trait.na}{Logical flag indicating whether \code{NA} values in the matrix of traits should be "corrected": NA values will be assigned the mean trait value.}
 \item{correct.trait}{Character. Either \code{"mean"} or \code{"exclude"}. Species missing in the data.frame of traits will be assigned mean trait values or  will be excluded from the analysis, respectively. }
  \item{correct.phylo}{Character. Either \code{"mean"} meaning \emph{"include missing species in the tree with a constant mean phylogenetic covariance"} or \code{"exclude"}, meaning \emph{"exclude missing species in the tree from the analysis"} }
  \item{comm}{A comunity data table (sites x species).}
 \item{phy}{A comunity data table (sites x species).}
  
}


\details{
\code{localdar} estimates any of the individual diversity area indices (isar, ipsvar,ipsrar, ipsear, ipscar, icwmar, icwmar.O,  iraodar or imntdar) at specific locations, such as the locations of trees of a "focal" point partern or at some grid of points (i.e., "mapping" local diversity area relationships). If no predefined focal point  pattern is provided (by the arguments \code{mippp.sp} or \code{mimark}), \code{localdar} will, by default, estimate the selected idar function in a 30 x 30 point-grid and return a map (the size of the grid can be modified by the arguments \code{nx} and \code{ny}).
\code{fdismap} and \code{raoDmap} are internal functions used by \code{localdar} to get the individual componentes of iraoD and FDis, instead of the averages obtained by \code{\link{envelope4idar}}.

}
\value{
If a focal point patternn has been provided by the arguments \code{mippp.sp} or \code{mimark}, \code{localdar} will return a list of marked point patterns (as long as the vector \code{r}) with the marks showing the estimation of the selected diversity index for the local community defined by a circle of radius \emph{r} around each of the points of the focal pattern. If no focal point pattern is provided, it will return a list of maps (as long as the vector \code{r})  each with the format \link[spatstat.geom]{im} of \pkg{spatstat}, each pixel showing the estimation of the selected diversity index for the local community defined by a circle of radius \emph{r} around the pixel center.
}

\author{ Marcelino de la Cruz \email{marcelino.delacruz@urjc.es}}

\seealso{ \code{\link{envelope4idar}}}

\examples{

# Map of local species area-relationship [ISAR(r)] in San Francisco plot at r=5 and r= 6 m
  data(SF)
  data(SFphylotree)
  data(SFtraits)

  isarSF <- localdar(SF, r=5:6, namesmark="species")
  
  isarSF
  
  plot(isarSF[[1]])
  
  
# Map of local species area-relationship [ISAR(r)] in San Francisco plot at r=5 and r= 6 m,
# with an adaptive buffer

isarSF <- localdar(SF, r=5:6, namesmark="species",buffer="adapt")



# Map of local species area-relationship [ISAR(r)] in lansing woods at different radii,
# with an fixed  buffer (only for rectangular windows).

data(lansing)
lansing.bfw<- owin(c(0.2,0.8),c(0.2,0.8))
lansing.lsar.bf<-localdar(lansing, r=seq(0.05,0.2,by=0.05), bfw=lansing.bfw)


# Map of local species area-relationship [ISAR(r)] in San Francisco plot at r=5 and r= 6 m,
# with a buffer of 7 m within plot limits

  mibfw<- erosion(SF$win, r=7)

 isarSF <- localdar(SF, r=5:6, namesmark="species", bfw=mibfw)



# Estimate local species area-relationship [ISAR(r)] in the local communities 
#  in circles with radius  r=5 and r= 6 m around the individuals of sp_44

  sp_44_sar <- localdar(SF, r=5:6, namesmark="species", mimark="sp_44")
  sp_44_sar
  plot(sp_44_sar[[1]])
  sp_44_sar[[1]]$marks
  


# Estimate local species area-relationship [ISAR(r)] in the local communities 
#  in circles with radius  r=5 and r= 6 m around the individuals of sp_44
# EXCLUDING the focal species from species counts.

  sp_44_sar <- localdar(SF, r=5:6, namesmark="species", mimark="sp_44", cross.idar=TRUE)
  sp_44_sar
  plot(sp_44_sar[[1]])
  sp_44_sar[[1]]$marks
 
  
  


# Map and estimation of local Phylogenetic Species Variety
local_psvar<-localdar(SF, r=5:6, idar="ipsvar", tree=SFphylotree, namesmark="species")
sp44_psvar <- localdar(SF, r=5:6, idar="ipsvar", tree=SFphylotree, namesmark="species",
                                    mimark="sp_44", buffer="adapt")

# Map and estimation of local Functional Dispersion
local_fdar <- localdar(SF, nx=50, ny=25, r=5:6,  idar="ifdar", traits=SFtraits,
                                 namesmark="species", correct.trait.na=TRUE)
sp44_fdar <- localdar(SF, nx=50, ny=25, r=5:6,  idar="ifdar", traits=SFtraits,
                                  namesmark="species", mimark="sp_44", correct.trait.na=TRUE)


# Map of a local community weighted mean of wood density
# first, put the wood desnsity data as a named vector
 wood.density.vec<-unlist(SFtraits[,  "wood.density", drop=FALSE])
    names(wood.density.vec)<- rownames(SFtraits)
    
local_cwd <- localdar(SF, r=5:6,  idar="icwmar", traits=wood.density.vec,
                                  namesmark="species", correct.trait.na=TRUE)
local_O.cwm <-localdar(SF, r=5:6, idar="icwmar.O", traits=wood.density.vec, 
                                     namesmark="species", correct.trait.na=TRUE)

# Map of Rao's phylogenetic diversity
local_rao<- localdar(SF,  r=5:6, idar="iraodar",  tree=SFphylotree, namesmark="species")
local_O.rao <- localdar(SF, r=5:6, idar="iraodar.O",  tree=SFphylotree, namesmark="species")

# Map of local mean nearest taxon distance 
local_mntd <- localdar(SF, r=5:6, idar="imntdar",  tree=SFphylotree, namesmark="species")


}

\keyword{ spatial }

\encoding{UTF-8}
\name{ipsim}
\alias{ipsim}
\alias{ipsimlist}
\alias{simulador2}
\alias{multifocalsimulator}

\title{Simulate Multivariate Point Patterns}
\description{
  These functions simulate  multivariate point patterns from a variety of null models, in the way required to test IDAR(r) functions.
}

\usage{
  ipsim(pp, mimark,sigma=0, lambda=NULL, namesmark=NULL)
  ipsimlist(pp, mimark, listsim)
  simulador2(mimark, milambda, nsim=99)
  multifocalsimulator(pp, mimark, simulate,nsim=99,nmin=NULL)
}

\arguments{
   \item{pp}{A multitype (a.k.a. multivariate) marked point pattern. An object with the \link[spatstat.geom]{ppp} format of \pkg{spatstat}}
   \item{mimark}{Character. Name of the focal species in the multitype \code{pp}.}
   \item{namesmark}{Character. If the marks in \code{pp} are within a \code{data.frame}, the name of the column with the species names}
   \item{sigma}{Sigma for the Gaussian kernel to estimate the intensity of the point pattern to simulate}
   \item{listsim}{List with simulated point paterns from \code{simulador2}}
   \item{lambda}{intensity surface, e.g., an image from density.ppp or predict.ppm}
   \item{milambda}{intensity surface, e.g., an image from density.ppp or predict.ppm}
   \item{nsim}{number of simulations}
   \item{nmin}{expected minimum number of points in each simulated pattern}
    \item{simulate}{either a list of pre-computed univariate point patterns or an expression in the R language indicating how to simulate the patterns.}
 }

\details{
This functions produce simulated point patterns appropriate to to compute envelopes of IDAR(r) functions. The usual tests of IDAR(r) functions require that the multivariate (i.e. multispecies) pattern remains fixed, except for the focal species, that is simulated acording to, e.g., an (inhomogeneous) Poisson process.

 \code{ipsim} returns the multivariate \code{pp} pattern with all species in the same locations except the "focal" one (i.e., the one indicated by the argument \code{mimark}) that is simulated using \link[spatstat.random]{rpoispp}. If  an intensity surface (argument \code{lambda}) is provided, the focal species will be simulated from this surface. If no \code{lambda} is provided but the argument \code{sigma} is >0, an intensity surface will be estimated with a Gaussian kernel with the sigma provided (using \link[spatstat.explore]{density.ppp}) and the simulation will be made form this surface. If no lambda is provided and \code{sigma}=0, a homogeneos Poisson process will be simulated.

 \code{simulador2} generates a list (length = \emph{nsim}) of marked (with mark = \emph{mimark}) univariate point patterns from an intensity surface using \link[spatstat.random]{rpoispp}.

 \code{ipsimlist} uses the results of \code{simulador2} and the multivariate \code{pp} pattern to generate a list of multivariate point patterns with all species in the same locations except the "focal" one (i.e., the one indicated by the argument \code{mimark}) that has the locations simulated with \code{simulador2}.
 
  \code{multifocalsimulator} is more flexible and allows the simulation of whichever null model of the focal species that could be described by an R expression.

}


\value{
  \code{ipsim} produces a multivariate point pattern (with the \link[spatstat.geom]{ppp} format of \pkg{spatstat}); \code{ipsimlist}  and \code{multifocalsimulator} produce a list of multivariate point patterns; \code{simulador2} produces a list of univariate marked patterns.
}

\author{ Marcelino de la Cruz \email{marcelino.delacruz@urjc.es}}
\examples{


# Build a multivariate point pattern where maple is simulated according to a Poisson process
# and where all the other species are keep fixed in their original coordinates.
# (The warnings are because in the original lansing point pattern there is a dupplicated
#  point)

data(lansing) 
ipsim(pp=lansing, mimark="maple")

# Build a multivariate point pattern where maple is simulated according to an Inhomogeneous
# Poisson process from an intensity surface estimated "on the fly" with a Gaussian kernel with
# sd = "sigma", and where all the other species are keep fixed in their original coordinates.

ipsim(pp=lansing, mimark="maple", sigma=0.1)

# Build a multivariate point pattern where maple is simulated according to an Inhomogeneous
# Poisson process from a predefined  intensity surface "lambda" and where all the other
# species are keep fixed in their original coordinates. "Lambda" is an im object resulting
# from density.ppp(), from predict.ppm() or converted from any other rasterized image. 

maple.lambda<- density.ppp(lansing[lansing$marks=="maple"])
ipsim(pp=lansing, mimark="maple", lambda=maple.lambda)

# Build a list of 19 multivariate point pattern where maple is simulated according to an
# Inhomogeneous Poisson process from a predefined  intensity surface "lambda" and where all
# the other species are keep fixed in their original coordinates. "Lambda" is an im object
# resulting  from density.ppp(), # from predict.ppm() or converted from any other rasterized
# image. 

  # Estimate the intensity of maple
   maple.lambda<- density(unmark(lansing[lansing$marks=="maple"]))

  # first simulate the individual maple patterns
   maple.sim<- simulador2(mimark="maple", milambda=maple.lambda, nsim=19)

  # Then, mix the simulated maple patterns with the rest of the multivariate pattern
  # (which remains "fixed")
   multi.maple.sim<- ipsimlist(pp=lansing, mimark="maple", listsim=maple.sim)

\dontrun{
## Use of multifocalsimulator() ##

# The same but in a single step with multifocalsimulator(): Build a list of 99 multivariate
# point pattern where maple is simulated according to an Inhomogeneous Poisson process
# from a predefined  intensity surface "lambda" and where all the other species are keep 
# fixed in their original coordinates. "Lambda" is an im object resulting from density.ppp(),
# from predict.ppm() or converted from any other rasterized image. 

   # Estimate the intensity of maple
   maple.lambda<- density(unmark(lansing[lansing$marks=="maple"]))

   # get 99 simulated multivariate point patterns where only maple varies,
   # according to an inhomogeneous Poisson process
   multi.maple.sim <- multifocalsimulator(lansing, "maple", nsim=99,
                                         simulate=expression(rpoispp(maple.lambda)))


   # Use the simulated multivariate patterns to compute  envelopes for the ISAR against
   # a null model of IPP for maple 
   isar.maple.env<- envelope(lansing, fun=isar, mimark="maple", nsim=99, savefuns=TRUE,
                                         r=seq(0.01, 0.25, le=100), simulate=multi.maple.sim)
   plot( isar.maple.env)



# Use multifocalsimulator()  to compute  envelopes for the ISAR against a null model of
# Poisson cluster for maple.

   # First, adjust a Poisson Cluster process to maple
   require(ecespa)
   maple.pc<- ipc.estK(unmark(lansing[lansing$marks=="maple"]))

  # generate list of simulated multivariate partterns (all other species fixed and maple
  # simulated according to the adjusted PC process):
   maple.pc.sim<-multifocalsimulator(pp=lansing, mimark="maple", nsim=99,nmin=NULL,
                                                 simulate=expression(rIPCP(maple.pc)))

  # compute envelopes
   isar.maple.pc.env<- envelope(lansing, fun=isar, mimark="maple", nsim=99, savefuns=TRUE, 
                                            simulate=maple.pc.sim, r=seq(0.01, 0.25, le=100))
   plot( isar.maple.pc.env)


# Compute  envelopes for the IFDAR against a null model of Poisson cluster for sp_44 in
# San Francisco forest.
   data(SF)
   data(SFtraits)

   # first, get the original point pattern but with marks only for the species (i.e., dsicard 
   # the data.frame of marks and keep only the vector of species names)
   SFsp<- unmark(SF)
   marks(SFsp)<- SF$marks$species

   # second, adjust a PCP to sp_44
   sp_44.pc<- ipc.estK(unmark(SFsp[SFsp$marks=="sp_44"]))

  # third, generate multivariate simulated patterns with only sp_44 varying according
  # to the adjusted PCP
   sp_44.pc.sim<-multifocalsimulator(pp=SFsp, mimark="sp_44",nsim=99,nmin=NULL,
                                                 simulate=expression(rIPCP(sp_44.pc)))

   # finally, compute envelopes
   ifdar.sp_44.pc.env<- envelope(SFsp, fun=ifdar, mimark="sp_44", traits=SFtraits, r=1:30,
                        correct.trait.na=TRUE, nsim=99, simulate=sp_44.pc.sim,  savefuns=TRUE)

   plot( ifdar.sp_44.pc.env)

}
}
\keyword{ spatial }

\name{fdisp}
\alias{fdisp}
\concept{multivariate dispersion}
\concept{functional dispersion}
\concept{functional diversity}
\title{Functional Dispersion}
\description{
\code{fdisp} measures the functional dispersion (FDis) of a set of communities, as described by \enc{Laliberté}{Laliberte} and Legendre (2010).
}
\usage{
fdisp(d, a, tol = 1e-07)
}
\arguments{
  \item{d}{a species-by- species distance matrix computed from functional traits, such as that returned by \code{\link{dist}} or \code{\link{gowdis}}. \code{\link{NA}s} are not allowed.}
  \item{a}{matrix containing the abundances of the species in \code{d} (or presence-absence, i.e. 0 or 1). Rows are sites and species are columns. Can be missing, in which case \code{fdisp} assumes that there is only one community with equal abundances of all species. \code{\link{NA}s} will be replaced by 0.
The number of species (columns) in \code{a} must match the number of species in \code{d}. In addition, the species labels in \code{a} and \code{d} must be identical and in the same order.}
  \item{tol}{tolerance threshold to test whether the distance matrix is Euclidean : an eigenvalue is considered positive if it is larger than -tol*\eqn{{\lambda}_1}{lambda1}, where \eqn{{\lambda}_1}{lambda1} is the largest eigenvalue.}
}
\details{
\code{fdisp} computes, for a set of communities, the average distance of individual objects (species) in PCoA space from any distance or dissimilarity measure, as described by Anderson (2006). The average distance to the centroid is a measure of multivariate dispersion and as been suggested as an index of beta diversity (Anderson et al. 2006). However, in \code{fdisp} both the centroid and the average distance to this centroid can be weighted by individual objects. In other words, \code{fdisp} returns the weighted average distance to the weighted centroid. This was suggested so that multivariate dispersion could be used as a multidimensional functional diversity (FD) index that can be weighted by species abundances. This FD index has been called functional dispersion (FDis) and is described by \enc{Laliberté}{Laliberte} and Legendre (2010).

In sum, FDis can account for relative abundances by shifting the position of the centroid towards the most abundant species, and then computing a weighted average distance to this new centroid, using again the relative abundances as weights (\enc{Laliberté}{Laliberte} and Legendre 2010). FDis has no upper limit and requires at least two species to be computed. For communities composed of only one species, \code{dbFD} returns a FDis value of 0. FDis is by construction unaffected by species richness, it can be computed from any distance or dissimilarity measure (Anderson et al. 2006), it can handle any number and type of traits (including more traits than species), and it is not strongly influenced by outliers.

FDis is conceptually similar to Rao's quadratic entropy Q (\enc{Botta-Dukát}{Botta-Dukat} 2005), and simulations (via \code{\link[FD]{simul.dbFD}}) have shown high positive correlations between the two indices (\enc{Laliberté}{Laliberte} and Legendre 2010). Still, one potential advantage of FDis over Rao's Q is that in the unweighted case (i.e. with presence-absence data), it opens possibilities for formal statistical tests for differences in FD between two or more communities through a distance-based test for homogeneity of multivariate dispersions (Anderson 2006); see \code{\link[vegan]{betadisper}} for more details.

Corrections for PCoA axes corresponding to negative eigenvalues are applied following Anderson (2006); see also \code{\link[vegan]{betadisper}} for more details on these corrections.
}
\value{
  \item{FDis}{vector listing the FDis of each community}
  \item{eig}{vector listing the eigenvalues of the PCoA}
  \item{vectors}{matrix containing the PCoA axes}
}
\references{
Anderson, M. J. (2006) Distance-based tests for homogeneity of multivariate dispersions. \emph{Biometrics} \bold{62}:245-253.

Anderson, M. J., K. E. Ellingsen and B. H. McArdle (2006) Multivariate dispersion as a measure of beta diversity. \emph{Ecology Letters} \bold{9}:683-693.

\enc{Botta-Dukát}{Botta-Dukat}, Z. (2005) Rao's quadratic entropy as a measure of functional diversity based on multiple traits. \emph{Journal of Vegetation Science} \bold{16}:533-540.

\enc{Laliberté}{Laliberte}, E. and P. Legendre (2010) A distance-based framework for measuring functional diversity from multiple traits. \emph{Ecology} \bold{91}299:305.
}
\author{Etienne \enc{Laliberté}{Laliberte} \email{etiennelaliberte@gmail.com} }
\note{
\code{fdisp} has been copied from former package \code{FD} as it was archived on April 15,2026.
}
\seealso{\code{\link[FD]{dbFD}} for computing multidimensional FD indices and \code{\link[vegan]{betadisper}} from which \code{fdisp} borrows some code.}

\keyword{multivariate}

\encoding{latin1}
\name{SF}
\alias{SF}
\alias{SFphylotree}
\alias{SFtraits}
\docType{data}
\title{
San Francisco forest plot.
}
\description{
Point pattern describing the locations of  822 trees (belonging to 113 species) in the San Francisco forest (southern Ecuador). Accompanied by a phylogenetic tree and a file with wood density of some of the species.
}
\usage{
data(SF)
data(SFphylotree)
data(SFtraits)
}
\format{
 \code{SF} is an object of class \code{ppp}  of \pkg{spatstat} representing the point pattern of trees locations, with a data.frame of marks. See \code{\link[spatstat.geom]{ppp.object}} 
  for details of the format.The dataset has 822 points with the following marks: 
  \describe{
    \item{species}{Species to which each tree belongs}
    \item{dbh}{Diameter at breast height of each tree}
      }
\code{SFphylotree} is a  phylogenetic tree of the class  \link[ape]{phylo} of \pkg{ape}, showing the phylogenetic relationshios among  296 tree species.

\code{SFtraits} is a data.frame with just one column, giving the estimated wood density for some of the 296 tree species.
}
\section{Details}{
This dataset represents the locations of trees with \emph{dbh} >= 5 cm in a polygonal plot of approx. 100 x 70 m in the montane forest near to Reserva \enc{Biológica}{Biologica} San Francisco  (Zamora-Chinchipe, southern Ecuador). These are part of the data collected by Vicuña (2016) and other Ecuadorian botanists and have been  analyzed several times ( e.g., \enc{Chacón}{Chacon} et al.  2014). The coordinates of trees are given in meters and the \emph{dbh} in cm. The phylogenetic tree has been extracted from Phylomatic and calibrated with the BLADJ algorithm of Phylocom. The data about wood density have been compiled from several sources and is expressed in \eqn{g/cm^3}.

}
\source{
\enc{Vicuña, R.  (2016) \emph{Estructura espacial y dinámica del bosque montano del Sur del Ecuador. Interacciones bióticas y limitaciones abióticas}. Tesis Doctoral. Universidad Politécnica de Madrid.}{Vicuna, R.  (2016) \emph{Estructura espacial y dinamica del bosque montano del Sur del Ecuador. Interacciones bioticas y limitaciones abioticas.} Tesis Doctoral. Universidad Politecnica de Madrid.}

}
\references{
\enc{Chacón-Labella et al. (2014) Negative density dependence and environmental heterogeneity effects on tree ferns across succession in a tropical montane forest.
\emph{Perspectives in Plant Ecology, Evolution and Systematics}  16(2): 52-63.}{Chacon-Labella et al. (2014) Negative density dependence and environmental heterogeneity effects on tree ferns across succession in a tropical montane forest. \emph{Perspectives in Plant Ecology, Evolution and Systematics}  16(2): 52-63.}
}

\keyword{datasets}
